/*
inSTREAM Version 4.2, October 2006.
Individual-based stream trout modeling software. Developed and maintained by Steve Railsback (Lang, Railsback & Associates, Arcata, California) and
Steve Jackson (Jackson Scientific Computing, McKinleyville, California).
Development sponsored by EPRI, US EPA, USDA Forest Service, and others.
Copyright (C) 2004 Lang, Railsback & Associates.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program (see file LICENSE); if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*/


#import <random.h>
#import <gui.h>
#import <objectbase/SwarmObject.h>
#import <stdlib.h>
#import "globals.h"
#import "Vector.h"
#import "TimeManagerProtocol.h"
#import "TroutModelSwarmP.h"
#import "StationObject.h"
#import "SurvMGRProtocol.h"
#import "InterpolationTableP.h"

//#define FOODAVAILREPORT


@interface Cell : SwarmObject
{
@protected
  id cellZone;

  id space;  // space of which Im a member

  id reach;  // The reach in which I belong; actually the space

  id myRandGen;

  id <TimeManager> timeManager;
  id <TroutModelSwarm> model;

  id <List> listOfAdjacentCells;

  int rasterResolution;
  int rasterResolutionX;
  int rasterResolutionY;
  char * rasterColorVariable;

  double velocity;
  double vertical;
  int transectNumber; 
  double myXDistance;
  int cellNumber;
  double cellDistToHide;
  Vector * refPoint;
  Vector * midPoint;
  Vector * extent;       // DIMENSION of percentages
  Vector ** boundary;    // numbers representing the bisection of each DIMENSION

  double **modelFlowPlan;
  int numStations;
  int numFlows;

  double depth;
  double area;

  int numberOfSpecies;
  id <Map> fishParamsMap;
  id <Map> survMgrMap;
  id <Map> survMgrReddMap;
  id <Symbol> ANIMAL;
  id <Symbol> HABITAT;

  double cellFracSpawn;
  double cellFracShelter;
  double cellShelterArea;
  double shelterAreaAvailable;
  BOOL   isShelterAvailable;


  id <List> fishIContain;
  id <List> reddsIContain;

  double driftHourlyCellTotal;
  double searchCellTotal;
  double searchHourlyCellTotal;
  double hourlyAvailDriftFood;
  double hourlyAvailSearchFood;

  double habShearParamA;
  double habShearParamB;
  double habShelterSpeedFrac;

  Color myColor;
  Color tagCellColor;
  BOOL tagCell;

  // 
  // REPORT VARIABLES
  //
  // BOOL foodReportFirstTime;  now in the habitat space
  //BOOL depthVelRptFirstTime;

  //
  // NEW HYDRAULIC FILE VARIABLES
  //
  id hydStation;
  double bottomElev;
  id <InterpolationTable> velocityInterpolator;
  id <InterpolationTable> wslInterpolator;


}
- createEnd;
- setSpace: aSpace;
- getSpace;

- setReach: aReach;
- getReach;


- setFishParamsMap: (id <Map>) aMap;
- setTimeManager: (id <TimeManager>) aTimeManager;
- setModel: (id <TroutModelSwarm>) aModel;
- setGenerator: aRandGen;
- getRandGen;
//
// Added for the survival manager
//
- setFishParamsMap: (id <Map>) aMap;
- setNumberOfSpecies: (int) aNumberOfSpecies;

- setHabShearParamA: (double) aShearParamA
     habShearParamB: (double) aShearParamB;

- (double) getHabShearParamA;
- (double) getHabShearParamB;

- setHabShelterSpeedFrac: (double) aShelterSpeedFrac;
- (double) getHabShelterSpeedFrac;

- setRasterResolution: (int) aResolution;
- setRasterResolutionX: (int) aResolutionX;
- setRasterResolutionY: (int) aResolutionY;
- (int) getRasterResolutionX;
- (int) getRasterResolutionY;
- (int) getRasterResolution;
- setRasterColorVariable: (char *) aColorVariable;

- setTransect: (int) aTransect;
- (int) getTransect;
- setCellNo: (int) aCellNo;
- (int) getCellNo;

- setMidPoint: (Vector *) aPoint;
- (Vector *) getMidPoint;
- setRefPoint: (Vector *) aPoint;    //the upper right corner of a cell
- (Vector *) getRefPoint; //Get the upper right corner of a cell
- (Vector *) getExtent;
- setExtent: (Vector *) aPoint;
- setBoundary: (Vector **) aVector;



//
// HYDRAULIC STATION
//
- setHydStation: aStation;
- checkThatHydStationIsNotNil;

- setVelocityInterpolator: (id <InterpolationTable>) anInterpolator;
- setWslInterpolator: (id <InterpolationTable>) anInterpolator;

- calcDepthAndVelocityWithIndex: (int) anInterpolationIndex
             withInterpFraction: (double) anInterpFraction;

- (Vector **) getBoundary;
- (double) getArea;
- (double) getXExtent;
- (double) getYExtent;

- (BOOL) containsX: (int) anInt;
- (BOOL) containsY: (int) anInt;

- (BOOL) containsDoubleX: (double) aDouble;
- (BOOL) containsDoubleY: (double) aDouble;

- drawSelfOn: (id <Raster>) aRaster;
- tagDestCells;

- getNeighborsWithin: (double) aRange
            withList: (id <List>) aCellList;
- (id <List>) getListOfAdjacentCells; 

- (int) getNumberOfFish;
- (id <List>) getFishIContain;
- (int)getNumberOfRedds;
- (id <List>) getReddsYouContain;

- setDepth: (double) aFloat;
- (double) getDepth;                   // return the depth
- (double) getFlowChange;

//- (double) getSpawnQualityForFish: aFish;

//SHELTER AREA
- (void) setCellFracShelter: (double) aFloat;
- (void) calcCellShelterArea;
- (double) getShelterAreaAvailable;
- (void) resetShelterAreaAvailable;
- (BOOL) getIsShelterAvailable;

- setCellFracSpawn: (double) aFloat;
- (double) getCellFracSpawn;
- (double) getCellFracShelter;

- spawnHere: aFish;
- eatHere: aFish;
- addFish: aFish;
- removeFish: aFish;
- addRedd: aRedd;
- removeRedd: aRedd;


- (double) getTemperature;
- (double) getTurbidity;
- (double) getDayLength;


- (double) getHabDriftConc;
- (double) getHabDriftRegenDist;
- (double) getHabPreyEnergyDensity;
- (double) getHabSearchProd;

- (double) getDistanceToHide;



// FOOD METHODS
-  calcDriftHourlyTotal;
- (double) getHourlyAvailDriftFood;
- calcSearchHourlyTotal;
- (double) getHourlyAvailSearchFood;


- setDepth: (double) aFloat;
- (double) getDepth;
- (BOOL) isDepthGreaterThan0;


// mortality risk mods

//SURVIVAL PROBABILITIES
- initializeSurvProb;
- updateHabitatSurvivalProb;
- updateHabSurvProbForAqPred;
- updateFishSurvivalProbFor: aFish;
- updateReddSurvivalProbFor: aRedd;
- (id <List>) getListOfSurvProbsFor: aFish;
- (id <List>) getReddListOfSurvProbsFor: aRedd;
- (double) getTotalKnownNonStarvSurvivalProbFor: aFish;
- (double) getStarvSurvivalFor: aFish;


- setCellVelocity : (double) newVelocity;
- (double) getCellVelocity;
- setDistanceToHide: (double) aDistance;

- (double) getYesterdaysRiverFlow;
- (double) getRiverFlow;            // These two get methods are
- (double) getTomorrowsRiverFlow;   // pass throughs to the habitatSpace

- (void) updateDSCellHourlyTotal;
- (void) resetAvailHourlyTotal;

#ifdef FOODAVAILREPORT
- foodAvailAndConInCell: aFish;
#endif

- depthVelReport: (FILE *) depthVelPtr;



//
// Barrier pass through to habitat space 
//
- (int) isThereABarrierTo: aCell;

//
// PRED DENSITY 
//
- (double) getPiscivorousFishDensity;



- (void) drop;

@end




