/*
inSTREAM Version 4.2, October 2006.
Individual-based stream trout modeling software. Developed and maintained by Steve Railsback (Lang, Railsback & Associates, Arcata, California) and
Steve Jackson (Jackson Scientific Computing, McKinleyville, California).
Development sponsored by EPRI, US EPA, USDA Forest Service, and others.
Copyright (C) 2004 Lang, Railsback & Associates.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program (see file LICENSE); if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*/


#include "HabitatManager.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


@implementation HabitatManager

+ createBegin: aZone
{
   HabitatManager* habManager;
   habManager = [super createBegin: aZone];

   habManager->siteLatitude = -LARGEINT;

   habManager->habitatSpaceNdx = nil;

   return habManager;
}



- createEnd
{
   return [super createEnd];
}


- setModel: aModel
{
   model = (id <TroutModelSwarm>) aModel;
   return self;
}


- setTimeManager: (id <TimeManager>) aTimeManager
{
   timeManager = aTimeManager;
   return self;
}


- setModelStartTime: (time_t) aRunStartTime
         andEndTime: (time_t) aRunEndTime
{
   runStartTime = aRunStartTime;
   runEndTime = aRunEndTime;
   return self;
}


- setDataStartTime: (time_t) aDataStartTime
        andEndTime: (time_t) aDataEndTime
{
   dataStartTime = aDataStartTime;
   dataEndTime = aDataEndTime;
   return self;
}


- setSiteLatitude: (double) aLatitude
{
   siteLatitude = aLatitude;
   return self;
}


///////////////////////////////////////
//
////      GET METHODS
//////
///////
///////////////////////////////////////

- (int) getNumberOfHabitatSpaces
{
      return numHabitatSpaces;
}


////////////////////////////////////
//
// getHabitatSpaceList
//
////////////////////////////////////
- getHabitatSpaceList
{
      return habitatSpaceList;
}


////////////////////////////////////
//
// getReachWithName
//
// It is the responsibility of the caller 
// to check for a nil return value.
//
////////////////////////////////////
- getReachWithName: (char *) aReachName
{
  
     id aHabitatSpace = nil;
     int i;

     for(i=0; i < numHabitatSpaces; i++)
     {
        aHabitatSpace = [habitatSpaceList atOffset: i]; 

        if(strcmp(aReachName, [aHabitatSpace getReachName]) == 0)
        {
           break;
        }

        aHabitatSpace = nil;
     }

     //if(aHabitatSpace == nil)
     //{
        //fprintf(stderr, "ERROR: HabitatManager >>>> getReachWithName >>>> There is no reach with name %s \n", aReachName);
        //fflush(0);
        //exit(1);
     //}

     return aHabitatSpace;

}




/////////////////////////////////////////////////////
//
// readReachSetupFile
//
/////////////////////////////////////////////////////
- readReachSetupFile: (char *) aReachSetupFile
{

  FILE * reachFilePtr =  NULL;

  char header[300];

  char reachVarName[50];
  char reachVar[50];

  id habitatSetup = nil;
  int habIndex = -1;

  fprintf(stdout, "HabitatManager >>>> readReachSetupFile >>>> BEGIN\n");
  fflush(0);
  

  if((reachFilePtr = fopen(aReachSetupFile, "r")) == NULL)
  {
      fprintf(stdout, "ERROR: HabitatManager >>>> readReachSetupFile >>>> fileName = %s \n", aReachSetupFile);
      fflush(0);
      exit(1);
  }

  fgets(header, 300, reachFilePtr);
  fgets(header, 300, reachFilePtr);
  fgets(header, 300, reachFilePtr);

      while(fscanf(reachFilePtr,"%s", reachVarName) != EOF)
      {
        if(strcmp(reachVarName, "REACHBEGIN") == 0)
        {
           fprintf(stdout, "HABITATMANAGER >>>> if reach begin >>>>\n");
           fflush(0);
           habitatSetup = [HabitatSetup createBegin: habManagerZone]; 
           [habitatSetup setHabitatIndex: ++habIndex];
           habitatSetup = [habitatSetup createEnd];
           [habitatSetupList addLast: habitatSetup];
           xprint(habitatSetup);
           xprint(habitatSetupList);
           continue;
        }
  fprintf(stdout, "HabitatManager >>>> readReachSetupFile >>>> while\n");
  fflush(0);
        if(strcmp(reachVarName, "REACHEND") == 0)
        {
           continue;
        }

        if(fscanf(reachFilePtr,"%s", reachVar) == EOF)
        {
            fprintf(stderr, "ERROR: HabitatManager >>>> End of file >>>> Please check the Habitat Manager Setup file\n");
            fflush(0);
            exit(1);
        }
    
        if(strcmp(reachVarName, "reachName") == 0)
        {
           [habitatSetup setReachName: reachVar];
           [habitatSetup setReachSymbol: [model getReachSymbolWithName: reachVar]];
        }
         
        if(strcmp(reachVarName, "habDownstreamJunctionNumber") == 0)
        {
           [habitatSetup setHabDStreamJNumber: atoi(reachVar)];
        }
         
        if(strcmp(reachVarName, "habUpstreamJunctionNumber") == 0)
        {
           [habitatSetup setHabUStreamJNumber: atoi(reachVar)];
        }
         
        if(strcmp(reachVarName, "habParamFile") == 0)
        {
           [habitatSetup setHabParamFile: reachVar];
        }
         
        if(strcmp(reachVarName, "cellDataFile") == 0)
        {
           [habitatSetup setCellDataFile: reachVar];
        }
         
        if(strcmp(reachVarName, "flowFile") == 0)
        {
           [habitatSetup setFlowFile: reachVar];
        }
         
        if(strcmp(reachVarName, "temperatureFile") == 0)
        {
           [habitatSetup setTemperatureFile: reachVar];
        }
         
        if(strcmp(reachVarName, "turbidityFile") == 0)
        {
           [habitatSetup setTurbidityFile: reachVar];
        }
         
        if(strcmp(reachVarName, "barrierX") == 0)
        {
           [habitatSetup setBarrierXWith: reachVar];
        }
         
        if(strcmp(reachVarName, "hydraulicsFile1") == 0)
        {
           [habitatSetup setHydraulicsFile1: reachVar];
        }
         
         
        if(strcmp(reachVarName, "hydraulicsFile2") == 0)
        {
           [habitatSetup setHydraulicsFile2: reachVar];
        }
         
         
        if(strcmp(reachVarName, "hydraulicsFile3") == 0)
        {
           [habitatSetup setHydraulicsFile3: reachVar];
        }
         
         
        if(strcmp(reachVarName, "hydraulicsFile4") == 0)
        {
           [habitatSetup setHydraulicsFile4: reachVar];
        }
         
         
        if(strcmp(reachVarName, "hydraulicsFile5") == 0)
        {
           [habitatSetup setHydraulicsFile5: reachVar];
        }
         
         
        fprintf(stdout,"reachVarName %s\n", reachVarName);
        fprintf(stdout,"reachVar %s\n", reachVar);
        xprint([habitatSetup getReachSymbol]);
        fflush(0);

      }

  fclose(reachFilePtr);

  numHabitatSpaces = [habitatSetupList getCount];

  fprintf(stdout, "HabitatManager >>>> readReachSetupFile >>>> END\n");
  fflush(0);


  return self;
  

}


- instantiateObjects
{

   habManagerZone = [Zone create: [self getZone]];
   habitatSetupList = [List create: habManagerZone];
   habitatSpaceList = [List create: habManagerZone];


  return self;

}


////////////////////////////////////////////
//
// setFishParamsMap
//
////////////////////////////////////////////
- setFishParamsMap: (id <Map>) aMap
{
    fishParamsMap = aMap;
    return self;
}



///////////////////////////////////////////////
//
// setNumberOfSpecies
//
////////////////////////////////////////////
- setNumberOfSpecies: (int) aNumberOfSpecies
{
   numberOfSpecies = aNumberOfSpecies;
   return self;
}


/////////////////////////////////////////////////////
//
// setRasterResolution
//
////////////////////////////////////////////////////
-        setRasterResolution: (int) aRasterResolution
        setRasterResolutionX: (int) aRasterResolutionX
        setRasterResolutionY: (int) aRasterResolutionY
      setRasterColorVariable: (char *) aRasterColorVariable
{

   rasterResolution = aRasterResolution;
   rasterResolutionX = aRasterResolutionX;
   rasterResolutionY = aRasterResolutionY;
   rasterColorVariable = aRasterColorVariable;

   return self;
}





///////////////////////////////////
//
// buildObjects
//
///////////////////////////////////
- buildObjects
{
   fprintf(stdout, "HabitatManager >>>> buildObjects >>>> BEGIN\n");
   fflush(0);

   [self finishBuildingTheHabitatSpaces];

   fprintf(stdout, "HabitatManager >>>> buildObjects >>>> END\n");
   fflush(0);
   return self;
}



/////////////////////////////////////////////////
//
// updateHabitatManagerWithTime
//
/////////////////////////////////////////////////
- updateHabitatManagerWithTime: (time_t) aTime
         andWithModelStartFlag: (BOOL) aStartFlag
{
   id habSpace = nil;
   int julianDate;

   modelTime = aTime;
   strncpy(modelDate, [timeManager getDateWithTimeT: modelTime], (size_t) 11);
   julianDate = [timeManager getJulianDayWithTimeT: modelTime]; 

   [solarManager updateDayLengthWithJulianDate: julianDate];

   if(habitatSpaceNdx == nil)
   {
        fprintf(stderr, "ERROR: HabitatManager >>>> updateHabitatManager... >>>> habitatSpaceNdx is nil\n");
        fflush(0);
        exit(1);
   }

   [habitatSpaceNdx setLoc: Start];
   while(([habitatSpaceNdx getLoc] != End) && ((habSpace = [habitatSpaceNdx next]) != nil))
   {
       [habSpace   updateHabitatWithTime: modelTime
                   andWithModelStartFlag: aStartFlag];
   }


   if(aStartFlag == NO)
   {
      #ifdef DEPTH_REPORT_ON
        [self printCellDepthReport];
      #endif 

      #ifdef VELOCITY_REPORT_ON
        [self printCellVelocityReport];
      #endif

      #ifdef HABITAT_REPORT_ON
        [self printHabitatReport];
      #endif

      #ifdef DEPTH_VEL_RPT
        [self printCellAreaDepthVelocityRpt];
      #endif

   }

   return self;

}


//////////////////////////////////////////
//
// createSolarManager
//
//////////////////////////////////////////
- createSolarManager
{
   if(siteLatitude == -LARGEINT)
   {
      fprintf(stderr, "ERROR: HabitatManager >>>>> siteLatitude hase not been set\n");
      fflush(0);
      exit(1);
   }

   solarManager = [SolarManager create: habManagerZone
                          withLatitude: siteLatitude
                      withHorizonAngle: 0.0
                    withTwilightLength: 6];

   return self;
}


////////////////////////////////////////////
//
// instantiateHabitatSpacesInZone
//
//
// The habitat space(s) must instantiated in the model zone
// so that the experiment swarm can talk to them.
//
////////////////////////////////////////////
- instantiateHabitatSpacesInZone: (id <Zone>) aZone
{
  HabitatSpace* habitatSpace = nil;
  int habitatSpaceCount;

  fprintf(stdout, "HabitatManager >>> instantiateHabitatSpacesInZone >>>> BEGIN\n");
  fflush(0); 
  
  for(habitatSpaceCount = 0; habitatSpaceCount < numHabitatSpaces; habitatSpaceCount++)
  {  

     id habSetup = [habitatSetupList atOffset: habitatSpaceCount];
 
     habitatSpace = [HabitatSpace createBegin: aZone];
     [habitatSpace setModel: model];
     [habitatSpace setNumberOfSpecies: numberOfSpecies];
     [ObjectLoader load: habitatSpace fromFileNamed: [habSetup getHabParamFile]];

     [habitatSpace setReachName: [habSetup getReachName]];
     [habitatSpace setInstanceName: [habSetup getReachName]];
     [habitatSpace setReachSymbol: [habSetup getReachSymbol]];
     [habitatSpace setHabDStreamJNumber: [habSetup getHabDStreamJNumber]];
     [habitatSpace setHabUStreamJNumber: [habSetup getHabUStreamJNumber]];

     [habitatSpace setFishParamsMap: fishParamsMap];
     [habitatSpace setSolarManager: solarManager];

     [habitatSpace buildObjects];

     [habitatSpace setCellDataFile: [habSetup getCellDataFile]];
     [habitatSpace setFlowFile: [habSetup getFlowFile]];
     [habitatSpace setTemperatureFile: [habSetup getTemperatureFile]];
     [habitatSpace setTurbidityFile: [habSetup getTurbidityFile]];
   
     //[habitatSpace setSpaceDimensions];
     //[habitatSpace setSizeX: [habitatSpace getSpaceDimX]  Y: [habitatSpace getSpaceDimY] ];

     habitatSpace->rasterResolution    = rasterResolution;
     habitatSpace->rasterResolutionX   = rasterResolutionX;
     habitatSpace->rasterResolutionY   = rasterResolutionY;
     habitatSpace->rasterColorVariable = rasterColorVariable;

     //habitatSpace = [habitatSpace createEnd];

     fprintf(stdout,"HabitatManager >>>> instantiateHabitatSpacesInZone >>>> read in the hydraulic files\n");
     fflush(0);

     /*
     //
     // read in the hydraulics files
     //
     if(strncmp([habSetup getHydraulicsFile1], "NONAME", strlen("NONAME")) == 0)
     {
         fprintf(stderr, "ERROR: HabitatManager >>>> hydraulicsFile1 has not been set");
         fflush(0);
         exit(1);
     }

     [habitatSpace  readHydraulicInputFile: [habSetup getHydraulicsFile1]];
  
     if(strncmp([habSetup getHydraulicsFile2], "NONAME", strlen("NONAME")) != 0)
     {
       [habitatSpace  readHydraulicInputFile: [habSetup getHydraulicsFile2]];
     } 
     if(strncmp([habSetup getHydraulicsFile3], "NONAME", strlen("NONAME")) != 0)
     {
       [habitatSpace  readHydraulicInputFile: [habSetup getHydraulicsFile3]];
     } 
     if(strncmp([habSetup getHydraulicsFile4], "NONAME", strlen("NONAME")) != 0)
     {
       [habitatSpace  readHydraulicInputFile: [habSetup getHydraulicsFile4]];
     } 
     if(strncmp([habSetup getHydraulicsFile5], "NONAME", strlen("NONAME")) != 0)
     {
       [habitatSpace  readHydraulicInputFile: [habSetup getHydraulicsFile5]];
     } 
     */

     [habitatSpace setBarrierRasterResolutionX: rasterResolutionX
                                andResolutionY: rasterResolutionY];


     xprint(habitatSpace);

     [habitatSpaceList addLast: habitatSpace];  

     xprint(habitatSpaceList);
  }

  //
  // This ndx exists for the lifetim of the modelrun 
  // DO NOT EVER DROP IT!
  //
  habitatSpaceNdx = [habitatSpaceList listBegin: habManagerZone];

  //[self buildReachJunctions];

  fprintf(stdout, "HabitatManager >>> instantiateHabitatSpacesInZone >>>> END\n");
  fflush(0); 

  return self;
}



/////////////////////////////////////////////
//
// finishBuildingTheHabitatSpaces
//
////////////////////////////////////////////
- finishBuildingTheHabitatSpaces
{

  HabitatSpace* habitatSpace = nil;
  int habitatSpaceCount;

  for(habitatSpaceCount = 0; habitatSpaceCount < numHabitatSpaces; habitatSpaceCount++)
  {  
     id habSetup = [habitatSetupList atOffset: habitatSpaceCount];

     habitatSpace = [habitatSpaceList atOffset: habitatSpaceCount];

     //
     // BEGIN: Code moved from instantiateHabitatSpaces
     //
 
     [habitatSpace setSpaceDimensions];
     [habitatSpace setSizeX: [habitatSpace getSpaceDimX]  Y: [habitatSpace getSpaceDimY] ];
     habitatSpace = [habitatSpace createEnd];

     //
     // read in the hydraulics files
     //
     if(strncmp([habSetup getHydraulicsFile1], "NONAME", strlen("NONAME")) == 0)
     {
         fprintf(stderr, "ERROR: HabitatManager >>>> hydraulicsFile1 has not been set");
         fflush(0);
         exit(1);
     }

     [habitatSpace  readHydraulicInputFile: [habSetup getHydraulicsFile1]];
  
     if(strncmp([habSetup getHydraulicsFile2], "NONAME", strlen("NONAME")) != 0)
     {
       [habitatSpace  readHydraulicInputFile: [habSetup getHydraulicsFile2]];
     } 
     if(strncmp([habSetup getHydraulicsFile3], "NONAME", strlen("NONAME")) != 0)
     {
       [habitatSpace  readHydraulicInputFile: [habSetup getHydraulicsFile3]];
     } 
     if(strncmp([habSetup getHydraulicsFile4], "NONAME", strlen("NONAME")) != 0)
     {
       [habitatSpace  readHydraulicInputFile: [habSetup getHydraulicsFile4]];
     } 
     if(strncmp([habSetup getHydraulicsFile5], "NONAME", strlen("NONAME")) != 0)
     {
       [habitatSpace  readHydraulicInputFile: [habSetup getHydraulicsFile5]];
     } 

     [self buildReachJunctions];

     //
     // END: Code moved from instantiateHabitatSpaces
     //


     [habitatSpace setTimeManager: timeManager];

     [habitatSpace setModelStartTime: runStartTime andEndTime: runEndTime];
     [habitatSpace setDataStartTime: dataStartTime andEndTime: dataEndTime];

     [habitatSpace createTimeSeriesInputManagers]; 

     habitatSpace = [habitatSpace createEnd];

     [habitatSpace buildCells];

     [habitatSpace createCellInterpolationTables];

     [habitatSpace setCellHydraulics];
  
     if([habSetup getBarrierXList] != nil)
     {
         [habitatSpace buildBarriersFromList: [habSetup getBarrierXList]];
     }

     [habitatSpace setBarrierRasterResolutionX: rasterResolutionX
                                andResolutionY: rasterResolutionY];

     [habitatSpace finishBuildObjects];
  }

  return self;
}


////////////////////////////////////////////////////
//
// buildHabSpaceCellFishInfoReporter
//
///////////////////////////////////////////////////
- buildHabSpaceCellFishInfoReporter
{
  HabitatSpace* habitatSpace = nil;
  int habitatSpaceCount;

  for(habitatSpaceCount = 0; habitatSpaceCount < numHabitatSpaces; habitatSpaceCount++)
  {  
     habitatSpace = [habitatSpaceList atOffset: habitatSpaceCount];
    [habitatSpace buildCellFishInfoReporter];
  }

  return self;
}

////////////////////////////////////////////////////////////
//
// buildReachJunctions
//
////////////////////////////////////////////////////////////

- buildReachJunctions
{

   id <ListIndex> reachNdx = [habitatSpaceList listBegin: scratchZone];
   id <ListIndex> reachJNdx = [habitatSpaceList listBegin: scratchZone];

   id aReach = nil;
   id aJReach = nil;

   [reachNdx setLoc: Start];

   while(([reachNdx getLoc] != End) && ((aReach = [reachNdx next]) != nil))
   {
      [reachJNdx setLoc: Start];

      while(([reachJNdx getLoc] != End) && ((aJReach = [reachJNdx next]) != nil))
      {
             if(aJReach == aReach) continue;
           
             if([aJReach getHabDStreamJNumber] == [aReach getHabDStreamJNumber])
             {
                 [aReach setHabDownstreamLinksToDS: aJReach];
             }
             
             if([aJReach getHabUStreamJNumber] == [aReach getHabDStreamJNumber])
             {
                 [aReach setHabDownstreamLinksToUS: aJReach];
             }
             
             if([aJReach getHabDStreamJNumber] == [aReach getHabUStreamJNumber])
             {
                 [aReach setHabUpstreamLinksToDS: aJReach];
             }
             
             if([aJReach getHabUStreamJNumber] == [aReach getHabUStreamJNumber])
             {
                 [aReach setHabUpstreamLinksToUS: aJReach];
             }
             
      }

   }

   [reachNdx drop];
   [reachJNdx drop];

   return self;
}



///////////////////////////////////////////////////////////
//
////
//////            FILE OUTPUT
////////
//////////
////////////
///////////////////////////////////////////////////////////


/////////////////////////////////////
//
// outputCellFishInfoReport
//
////////////////////////////////////
- outputCellFishInfoReport
{
   //fprintf(stdout, "HabitatManager >>>> outputCellFishInfoReport >>>> BEGIN\n");
   //fflush(0);

   [habitatSpaceList forEach: M(outputCellFishInfoReport)];

   //fprintf(stdout, "HabitatManager >>>> outputCellFishInfoReport >>>> END\n");
   //fflush(0);
   return self;
}
   



#ifdef DEPTH_REPORT_ON
- printCellDepthReport
{

   [habitatSpaceList forEach: M(printCellDepthReport)]; 

   return self;

}
    
#endif

#ifdef VELOCITY_REPORT_ON
- printCellVelocityReport
{

   [habitatSpaceList forEach: M(printCellVelocityReport)]; 

   return self;
}

#endif


#ifdef HABITAT_REPORT_ON
- printHabitatReport
{

   [habitatSpaceList forEach: M(printHabitatReport)]; 

   return self;
}
#endif

#ifdef DEPTH_VEL_RPT
- printCellAreaDepthVelocityRpt
{

   [habitatSpaceList forEach: M(printCellAreaDepthVelocityRpt)]; 

   return self;
}
#endif




@end
