/*
inSTREAM Version 4.2, October 2006.
Individual-based stream trout modeling software. Developed and maintained by Steve Railsback (Lang, Railsback & Associates, Arcata, California) and
Steve Jackson (Jackson Scientific Computing, McKinleyville, California).
Development sponsored by EPRI, US EPA, USDA Forest Service, and others.
Copyright (C) 2004 Lang, Railsback & Associates.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program (see file LICENSE); if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*/


#import "HabitatSetup.h"
#import <string.h>

@implementation HabitatSetup

+ createBegin: aZone
{

   HabitatSetup* habitatSetup = [super createBegin: aZone];

   habitatSetup->habIndex = -1;
  
   strcpy(habitatSetup->reachName,"NONAME");
   habitatSetup->reachSymbol = nil;
   strcpy(habitatSetup->habParamFile,"NONAME");
   strcpy(habitatSetup->cellDataFile,"NONAME");
   strcpy(habitatSetup->flowFile,"NONAME");
   strcpy(habitatSetup->temperatureFile,"NONAME");

   strcpy(habitatSetup->hydraulicsFile1,"NONAME");
   strcpy(habitatSetup->hydraulicsFile2,"NONAME");
   strcpy(habitatSetup->hydraulicsFile3,"NONAME");
   strcpy(habitatSetup->hydraulicsFile4,"NONAME");
   strcpy(habitatSetup->hydraulicsFile5,"NONAME");
   
   habitatSetup->barrierXList = nil;

   return habitatSetup;

}


- createEnd
{
   if(habIndex < 0)
   {
       fprintf(stderr, "HabitatSetup >>>> createEnd >>>> improper habIndex\n");
       fflush(0);
       exit(1);
   }

   return [super createEnd];

}


- setHabitatIndex: (int) anIndex
{
    habIndex = anIndex;
    return self;
}



- setHabDStreamJNumber: (int) aJunctionNum
{

     habDownStreamJunctionNumber = aJunctionNum;
     return self;
}
 


- setHabUStreamJNumber: (int) aJunctionNum
{
     habUpStreamJunctionNumber = aJunctionNum;
     return self;
}



- setReachName: (char *) aReachName
{
   strcpy(reachName, aReachName);

   fprintf(stdout,"HabitatSetup >>>> ReachName = %s \n", reachName);
   fflush(0);

   return self;
}


- setReachSymbol: (id <Symbol>) aReachSymbol
{
    reachSymbol = aReachSymbol;
    return self;
}



- setHabParamFile: (char *) aHabParamFile
{

  strcpy(habParamFile, aHabParamFile);

  fprintf(stdout,"HabitatSetup >>>> HabParamFile = %s \n", habParamFile);
  fflush(0);


  return self;

}



- setCellDataFile: (char *) aCellDataFile
{

   strcpy(cellDataFile, aCellDataFile);

   fprintf(stdout,"HabitatSetup >>>> CellDataFile = %s \n", cellDataFile);
   fflush(0);


   return self;

}



- setFlowFile: (char *) aFlowFile
{
   strcpy(flowFile, aFlowFile);
   fprintf(stdout,"HabitatSetup >>>> FlowFile = %s \n", flowFile);
   return self;
}


- setTemperatureFile: (char *) aTemperatureFile
{
   strcpy(temperatureFile, aTemperatureFile);
   fprintf(stdout,"HabitatSetup >>>> TemperatureFile = %s \n", temperatureFile);
   fflush(0);
   return self;
}



- setTurbidityFile: (char *) aTurbidityFile
{
   strcpy(turbidityFile, aTurbidityFile);
   fprintf(stdout,"HabitatSetup >>>> TurbidityFile = %s \n", turbidityFile);
   fflush(0);
   return self;
}


- setBarrierXWith: (char *) aBarrierX
{
   double* barrierX = (double *) [ZoneAllocMapper allocBlockIn: [self getZone]
                                                      ofSize: sizeof(double)];
   if(barrierXList == nil)
   {
      barrierXList = [List create: [self getZone]];
   }

   *barrierX = atof(aBarrierX);
   [barrierXList addLast: (void *) barrierX];
   return self;
}


- setHydraulicsFile1: (char *) aHydraulicsFile
{

   strcpy(hydraulicsFile1, aHydraulicsFile);

   fprintf(stdout,"HabitatSetup >>>> HydraulicsFile1 = %s \n", hydraulicsFile1);
   fflush(0);

   return self;

}




- setHydraulicsFile2: (char *) aHydraulicsFile
{

   strcpy(hydraulicsFile2, aHydraulicsFile);

   fprintf(stdout,"HabitatSetup >>>> HydraulicsFile2 = %s \n", hydraulicsFile2);
   fflush(0);


   return self;

}




- setHydraulicsFile3: (char *) aHydraulicsFile
{

   strcpy(hydraulicsFile3, aHydraulicsFile);

   fprintf(stdout,"HabitatSetup >>>> HydraulicsFile3 = %s \n", hydraulicsFile3);
   fflush(0);


   return self;

}




- setHydraulicsFile4: (char *) aHydraulicsFile
{

   strcpy(hydraulicsFile4, aHydraulicsFile);

   fprintf(stdout,"HabitatSetup >>>> HydraulicsFile4 = %s \n", hydraulicsFile4);
   fflush(0);


   return self;

}




- setHydraulicsFile5: (char *) aHydraulicsFile
{

   strcpy(hydraulicsFile5, aHydraulicsFile);

   fprintf(stdout,"HabitatSetup >>>> HydraulicsFile5 = %s \n", hydraulicsFile5);
   fflush(0);


   return self;

}



- (int) getHabitatIndex
{
   return habIndex;

}


- (char *) getReachName
{
   return reachName;
}


- (id <Symbol>) getReachSymbol
{
   return reachSymbol;
}


- (int) getHabDStreamJNumber
{
    return habDownStreamJunctionNumber;
}

- (int) getHabUStreamJNumber
{
    return habUpStreamJunctionNumber;
}




- (char *) getHabParamFile
{
   return habParamFile;
}


- (char *) getCellDataFile
{
   return cellDataFile;
}


- (char *) getFlowFile
{
   return flowFile;
}


- (char *) getTemperatureFile
{
   return temperatureFile;
}

- (char *) getTurbidityFile
{
   return turbidityFile;

}


- (id <List>) getBarrierXList
{
   return barrierXList;
}


- (char *) getHydraulicsFile1
{
   return hydraulicsFile1;
}


- (char *) getHydraulicsFile2
{
   return hydraulicsFile2;
}


- (char *) getHydraulicsFile3
{
   return hydraulicsFile3;
}


- (char *) getHydraulicsFile4
{
   return hydraulicsFile4;
}


- (char *) getHydraulicsFile5
{
   return hydraulicsFile5;
}


@end
