/*
inSTREAM Version 4.2, October 2006.
Individual-based stream trout modeling software. Developed and maintained by Steve Railsback (Lang, Railsback & Associates, Arcata, California) and
Steve Jackson (Jackson Scientific Computing, McKinleyville, California).
Development sponsored by EPRI, US EPA, USDA Forest Service, and others.
Copyright (C) 2004 Lang, Railsback & Associates.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program (see file LICENSE); if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*/




#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#import <math.h>
#import <random.h>
#import "globals.h"
#import "Vector.h"
#import "HabitatSpace.h"


@implementation HabitatSpace
+ createBegin: aZone 
{
  HabitatSpace* obj;

  obj = [super createBegin: aZone];


  obj->tomorrowsRiverFlow = -LARGEINT;
  obj->yesterdaysRiverFlow = -LARGEINT;

  obj->flowInputManager = nil;

  obj->hydraulicFileReader = nil;

  obj->solarManager = nil;

  obj->habitatRptFilePtr = NULL;

  obj->appendFiles = NO;
  obj->scenario = 0;
  obj->replicate = 0;

  obj->cellFishInfoReporter = nil;
  obj->ageSymbolList = nil;
  obj->speciesSymbolList = nil;

  obj->cellFishList = nil;    

  obj->reachName = NULL;
  obj->instanceName = NULL;

  obj->habitatZone = [Zone create: aZone];

  obj->flowFile = (char *) [ZoneAllocMapper allocBlockIn: obj->habitatZone
                                                ofSize: 50*sizeof(char)];

  obj->temperatureFile = (char *) [ZoneAllocMapper allocBlockIn: obj->habitatZone
                                                         ofSize: 50*sizeof(char)];

  obj->turbidityFile = (char *) [ZoneAllocMapper allocBlockIn: obj->habitatZone
                                                       ofSize: 50*sizeof(char)];

  obj->hydraulicsFile = (char *) [ZoneAllocMapper allocBlockIn: obj->habitatZone
                                                        ofSize: 50*sizeof(char)];

  obj->cellDataFile = (char *) [ZoneAllocMapper allocBlockIn: obj->habitatZone
                                                      ofSize: 50*sizeof(char)];

  //
  // Initialize the habitat paramters to nonsense values
  //
  obj->habDriftConc          = -1.0;
  obj->habDriftRegenDist     = -1.0;
  obj->habMaxSpawnFlow       = -1.0;
  obj->habPreyEnergyDensity  = -1.0;
  obj->habSearchProd         = -1.0;
  obj->habShearParamA        = -99999999999.0;
  obj->habShearParamB        = -99999999999.0;
  obj->habShelterSpeedFrac   = -1.0;

  return obj;
}

/////////////////////////////////////////////
//
// createEnd
//
////////////////////////////////////////////
- createEnd {

  [super createEnd];

  temperature = LARGEINT;

  [self checkHabitatParams];

  return self;
}



///////////////////////////////////////////////
//
// checkHabitatParams
//
///////////////////////////////////////////////
- checkHabitatParams
{
   BOOL ERROR = NO;

   if(habDriftConc  < 0.0)
   {
      ERROR = YES;
   }          
   if(habDriftRegenDist < 0.0)
   {
      ERROR = YES;
   }
   if(habMaxSpawnFlow < 0.0)
   {
      ERROR = YES;
   }
   if(habPreyEnergyDensity < 0.0)
   {
      ERROR = YES;
   }
   if(habSearchProd < 0.0)
   {
      ERROR = YES;
   }
   if(habShearParamA == -99999999999.0)
   {
      ERROR = YES;
   }
   if(habShearParamB == -99999999999.0)
   {
      ERROR = YES;
   }
   if((habShelterSpeedFrac < 0) || habShelterSpeedFrac > 1.0)
   {
       ERROR = YES;
   }

   if(ERROR)
   {
       fprintf(stderr, "ERROR: HabitatSpace >>>> checkHabitatParams >>>> reachName %s >>>> please check the inputted parameters for correct or missing values\n", reachName);
       fflush(0);
       exit(1); 
   }

   return self;
}



////////////////////////////////////////////////
//
// buildObjects
//
///////////////////////////////////////////////
- buildObjects 
{


 modelDate = (char *) [ZoneAllocMapper allocBlockIn: habitatZone
                                             ofSize: 12 * sizeof(char)];

 barrierList = [List create: habitatZone];

 habitatReportFirstWrite = YES;
 depthReportFirstWrite = YES;
 velocityReportFirstWrite = YES;
 depthVelRptFirstTime=YES;

 foodReportFirstTime = YES;

  //
  // BEGIN REPORT FILES
  //
  cellDepthReportFile = (char *) [ZoneAllocMapper allocBlockIn: habitatZone
                                        ofSize: (strlen(reachName) + strlen("Cell_Flow_Depth_Test.rpt") + 1)];

  strcpy(cellDepthReportFile, reachName);
  strcat(cellDepthReportFile, "Cell_Flow_Depth_Test.rpt"); 

  cellVelocityReportFile = (char *) [ZoneAllocMapper allocBlockIn: habitatZone
                                        ofSize: (strlen(reachName) + strlen("Cell_Flow_Velocity_Test.rpt") + 1)];

  strcpy(cellVelocityReportFile, reachName);
  strcat(cellVelocityReportFile, "Cell_Flow_Velocity_Test.rpt"); 

  habitatReportFile = (char *) [ZoneAllocMapper allocBlockIn: habitatZone
                                        ofSize: (strlen(reachName) + strlen("Habitat.rpt") + 1)];

  strcpy(habitatReportFile, reachName);
  strcat(habitatReportFile, "Habitat.rpt"); 

  cellAreaDepthVelReportFile = (char *) [ZoneAllocMapper allocBlockIn: habitatZone
                                        ofSize: (strlen(reachName) + strlen("CellDepthAreaVelocity.rpt") + 1)];

  strcpy(cellAreaDepthVelReportFile, reachName);
  strcat(cellAreaDepthVelReportFile, "CellDepthAreaVelocity.rpt"); 
  
  // 
  // END REPORT FILES
  //

  Date = (char *) [ZoneAllocMapper allocBlockIn: habitatZone ofSize: 12*sizeof(char)];

  habDownstreamLinksToDS = [List create: habitatZone]; 
  habDownstreamLinksToUS = [List create: habitatZone];

  habUpstreamLinksToDS = [List create: habitatZone];
  habUpstreamLinksToUS = [List create: habitatZone];

  tempCellList = [List create: habitatZone];

  if(solarManager == nil)
  {
     fprintf(stderr, "ERROR: HabitatSpace >>>> buildObjects >>>> solarManager has not been set\n");
     fflush(0);
     exit(1);
  }

  cellFishList = [List create: habitatZone];

  return self;

} //buildObjects

////////////////////////////////////////////
//
// finishBuildObjects
//
////////////////////////////////////////////
- finishBuildObjects
{

   ageSymbolList = [model getAgeSymbolList];
   speciesSymbolList = [model getSpeciesSymbolList];

   scenario = [model getScenario];
   replicate = [model getReplicate];

   appendFiles = [model getAppendFiles];

   return self;
}

///////////////////////////////////////////////////////////////
//
// setModel
//
//////////////////////////////////////////////////////////////
- setModel: (id <TroutModelSwarm>) aModel 
{
  model = aModel;
  return self;
}


////////////////////////////////////////////////
//
// setSolarManager
//
////////////////////////////////////////////////
- setSolarManager: (id <SolarManager>) aSolarManager
{
   solarManager = aSolarManager;
   return self;
}

///////////////////////////////////////
//
// setNumberOfSpecies
//
//////////////////////////////////////
- setNumberOfSpecies: (int) aNumberOfSpecies
{
   numberOfSpecies = aNumberOfSpecies;
   return self;
}
////////////////////////////////////////////////////////////
//
// getModel
//
///////////////////////////////////////////////////////////
- getModel 
{
  return model;
}


////////////////////////////////////////
//
// setReachName
//
///////////////////////////////////////
- setReachName: (char *) aReachName
{
   
   size_t strLength = strlen(aReachName); 

   if(reachName == NULL)
   {
       reachName = (char *) [ZoneAllocMapper allocBlockIn: habitatZone
                                                   ofSize: strLength * sizeof(char) + 1];
   }
   else
   {
       fprintf(stderr, "ERROR: HabitatSpace >>>> setReachName >>>> attempting to set reach name more than once\n");
       fflush(0);
       exit(1);
   }

   strncpy(reachName, aReachName, strLength + 1);

   return self;
}


////////////////////////////////////////////////
//
// setInstanceName
//
////////////////////////////////////////////////
- setInstanceName: (char *) anInstanceName
{
   size_t strLength = strlen(anInstanceName); 

   if(instanceName == NULL)
   {
       instanceName = (char *) [ZoneAllocMapper allocBlockIn: habitatZone
                                                   ofSize: strLength * sizeof(char) + 1];
   }
   else
   {
       fprintf(stderr, "ERROR: HabitatSpace >>>> setInstanceName >>>> attempting to set instance name more than once\n");
       fflush(0);
       exit(1);
   }

   strncpy(instanceName, anInstanceName, strLength + 1);

   return self;
}

////////////////////////////////////
//
// getReachName
//
////////////////////////////////////
- (char *) getReachName
{
    return reachName;
}


///////////////////////////////////////
//
// getInstanceName
//
//////////////////////////////////////
- (char *) getInstanceName
{
    if(instanceName == NULL)
    {
         fprintf(stderr, "ERROR: HabitatSpace >>>> getInstanceName >>>> instanceName is NULL\n");
         fflush(0);
         exit(1);
    }
    return instanceName;
}

//////////////////////////////
//
// setReachSymbol
//
/////////////////////////////
- setReachSymbol: (id <Symbol>) aReachSymbol
{
   reachSymbol = aReachSymbol;
   return self;
}

///////////////////////////////
//
// getReachSymbol
//
///////////////////////////////
- (id <Symbol>) getReachSymbol
{
    return reachSymbol;
}



////////////////////////////////////////////
//
////
//////          ADJACENT REACHES
////////
/////////
////////////////////////////////////////////

/////////////////////////////////////////////
//
// JUNCTION NUMBERS 
//
////////////////////////////////////////////

- setHabDStreamJNumber: (int) aJunctionNumber
{
    habDownstreamJunctionNumber = aJunctionNumber;

    return self;
}

- setHabUStreamJNumber: (int) aJunctionNumber
{
    habUpstreamJunctionNumber = aJunctionNumber;
    return self;
}

- (int) getHabDStreamJNumber
{
     return habDownstreamJunctionNumber;
}


- (int) getHabUStreamJNumber
{
     return habUpstreamJunctionNumber;
}




- setHabDownstreamLinksToDS: aDSLinkToDS
{

   [habDownstreamLinksToDS addLast: aDSLinkToDS];

   return self;
}


- setHabDownstreamLinksToUS: aDSLinkToUS
{

   [habDownstreamLinksToUS addLast: aDSLinkToUS];

   return self;
}


- setHabUpstreamLinksToDS: anUSLinkToDS
{

   [habUpstreamLinksToDS addLast: anUSLinkToDS];

   return self;
}


- setHabUpstreamLinksToUS: anUSLinkToUS
{

   [habUpstreamLinksToUS addLast: anUSLinkToUS];

   return self;
}



- (id <List>) getHabDownstreamLinksToDS
{
   return habDownstreamLinksToDS;
}


- (id <List>) getHabDownstreamLinksToUS
{
   return habDownstreamLinksToUS;
}


- (id <List>) getHabUpstreamLinksToDS
{
   return habUpstreamLinksToDS;
}


- (id <List>) getHabUpstreamLinksToUS
{
   return habUpstreamLinksToUS;
}











/////////////////////////////////////////////////////
//
// setTimeManager
//
/////////////////////////////////////////////////////
- setTimeManager: (id <TimeManager>) aTimeManager
{
      timeManager = aTimeManager;

      return self;
}


////////////////////////////////////////////////////
//
// setFishParamsMap
//
////////////////////////////////////////////////////
- setFishParamsMap: (id <Map>) aMap
{
    fishParamsMap = aMap;
    return self;
}


/////////////////////////////////////////////////////////////
//
// getCells
//
////////////////////////////////////////////////////////////
- (id <List>) getCells {
  return listOfCells;
}



/////////////////////////////////////////
//
// set Input Files
//
////////////////////////////////////////
- setCellDataFile: (char*) aFile 
{
    strncpy(cellDataFile, aFile, (size_t) 50);
    return self;
}

- setFlowFile: (char*) aFile 
{
   strncpy(flowFile, aFile, (size_t) 50);
   return self;
}

- setTemperatureFile: (char*) aFile 
{
    strncpy(temperatureFile, aFile, (size_t) 50);
    return self;
}


/////////////////////////////////////////////////////////////////
//
// setTurbidityFile
//
//////////////////////////////////////////////////////////////
- setTurbidityFile: (char*) aFile 
{
  strncpy(turbidityFile, aFile, (size_t) 50);
  return self;
}


///////////////////////////////////////////////
//
// createTimeSeriesInputManagers
//
///////////////////////////////////////////////
- createTimeSeriesInputManagers
{
   flowInputManager = [TimeSeriesInputManager  createBegin: habitatZone
                                              withDataType: "DAILY"
                                             withInputFile: flowFile
                                           withTimeManager: timeManager
                                             withStartTime: dataStartTime
                                               withEndTime: dataEndTime
                                             withCheckData: NO];

   flowInputManager = [flowInputManager createEnd];
 
   temperatureInputManager = [TimeSeriesInputManager  createBegin: habitatZone
                                                     withDataType: "DAILY"
                                                    withInputFile: temperatureFile
                                                  withTimeManager: timeManager
                                                    withStartTime: dataStartTime
                                                      withEndTime: dataEndTime
                                                    withCheckData: NO];

   temperatureInputManager = [temperatureInputManager createEnd];

   turbidityInputManager = [TimeSeriesInputManager  createBegin: habitatZone
                                                   withDataType: "DAILY"
                                                  withInputFile: turbidityFile
                                                withTimeManager: timeManager
                                                  withStartTime: dataStartTime
                                                    withEndTime: dataEndTime
                                                  withCheckData: NO];

    turbidityInputManager = [turbidityInputManager createEnd];

    return self;
}




////////////////////////////////////////////////////////////////
//
// setSpaceDimensions
// uses the geometry file to set the space dimensions
//
////////////////////////////////////////////////////////////
- setSpaceDimensions 
{
  FILE * geomFP=NULL;
  char *header1;
  int numberOfItemsRead = 0;
  int transect = -1;
  int cellno = -1;
  double upstream=-1.0, lefty=-1.0, fracshel=-1.0, distohi=-1.0, fracspawn=-1.0;
  int maxTransect = -1;
  int maxCellNo = -1;

  char inputString[300];
  char inputTransect[10];
  char inputCellNo[10];
  char inputUpStream[10];
  char inputLeftY[10];
  char inputFracShel[10];
  char inputDistToHide[10];
  char inputFracSpawn[10];

  BOOL ERROR = FALSE;

  header1 = (char *)malloc(HCOMMENTLENGTH*sizeof(char));

  if((geomFP = fopen(cellDataFile, "r")) == NULL) 
  {
     fprintf(stderr, "ERROR: HabitatSpace >>>> setSpaceDimensions >>>> Cannot open file %s\n",cellDataFile);
     fflush(0);
     exit(1);
  }
  else 
  {
   fgets(header1,HCOMMENTLENGTH,geomFP);
   fgets(header1,HCOMMENTLENGTH,geomFP);
   fgets(header1,HCOMMENTLENGTH,geomFP);
  
   while(fgets(inputString, 300, geomFP) != NULL)
   {
      numberOfItemsRead = sscanf(inputString, "%s %s %s %s %s %s %s",  inputTransect,
                                                   inputCellNo,
                                                   inputUpStream,
                                                   inputLeftY,
                                                   inputFracShel,
                                                   inputDistToHide,
                                                   inputFracSpawn);
      if(numberOfItemsRead == EOF)
      {
         break;
      }

      if(numberOfItemsRead != 7) 
      {
         ERROR = TRUE;
         break;
      }

      transect = atoi(inputTransect);
      cellno = atoi(inputCellNo);
      upstream = atof(inputUpStream);
      lefty = atof(inputLeftY);
      fracshel = atof(inputFracShel);
      distohi = atof(inputDistToHide);
      fracspawn = atof(inputFracSpawn);

      if(   (transect < 0)
         || (cellno < 0)
         || (upstream < 0.0)
         || (lefty < 0.0)
         || (fracshel < 0.0)
         || (distohi < 0.0)
         || (fracspawn < 0.0))
      {
          ERROR = TRUE;
          break;
      }

      strncpy(inputTransect, "\0" , 10);
      strncpy(inputCellNo, "\0" , 10);
      strncpy(inputUpStream, "\0" , 10);
      strncpy(inputLeftY, "\0" , 10);
      strncpy(inputFracShel, "\0" , 10);
      strncpy(inputDistToHide, "\0" , 10);
      strncpy(inputFracSpawn, "\0" , 10);

      maxTransect = (maxTransect > transect) ? maxTransect : transect;
      maxCellNo = (maxCellNo > cellno) ? maxCellNo : cellno;
   }

  spaceDimX = maxTransect;
  spaceDimY = maxCellNo;
  }

  if(ERROR == TRUE)
  {
     fprintf(stderr, "ERROR: HabitatSpace >>>> setSpaceDimensions >>>> checkcellDataFile for correct entries\n");
     fflush(0);
     exit(1);
  } 

  fclose(geomFP);
  free(header1);

  return self;
}



////////////////////////////////////////////////////////
//
// getSpaceDimX and getSpaceDimY
//
//////////////////////////////////////////////////////
- (int) getSpaceDimX 
{
   return spaceDimX;
}

- (int) getSpaceDimY 
{
   return spaceDimY;
}


////////////////////////////////////////////////////////////////
//
// readGeometry
//
// Coordinates are converted from input in meters 
// to centimeters in this method
//
///////////////////////////////////////////////////////////////
- readGeometry 
{
  FILE * geomFP=NULL;
  char * header1;
  int transect=1;
  int oldtransect=1;
  int prevTransect=0;
  int cellno=0;
  int prevCellno=0;
  double upstream=0.0, lefty=0.0, fracshel=0.0, disttohi=0.0, fracspawn=0.0;
  double prevupstream=0.0,oldupstream=0.0, prevlefty=0.0;
  double xExtent;
  double yExtent;
  BOOL checkTransect = YES;

  header1 = (char *) [[self getZone] alloc: HCOMMENTLENGTH*sizeof(char)];

 if(( geomFP = fopen(cellDataFile, "r")) == NULL )
 {
   fprintf(stderr, "ERROR: HabitatSpace >>>> readGeometry >>>> Cannot open file %s\n",cellDataFile);
   fflush(0);
   exit(1);
 }
 else 
 {
  listOfCells = [List create: [self getZone]];

  fgets(header1,HCOMMENTLENGTH,geomFP);
  fgets(header1,HCOMMENTLENGTH,geomFP);
  fgets(header1,HCOMMENTLENGTH,geomFP);


  //
  // Error checking on the data file was performed in setSpaceDimensions 
  //

   while(fscanf(geomFP,"%d %d %lf %lf %lf %lf  %lf",
             &transect, &cellno,&upstream, &lefty,
             &fracshel, &disttohi,&fracspawn) != EOF) 
  {
        Cell* cell;
        double midPointX;
        double midPointY;
          
        if(transect == 1 && oldtransect == 1) 
        {
           oldupstream = upstream;
        }
  
        if(cellno == 1)
        {
            prevCellno = 1;
        }
        else if(cellno != (prevCellno + 1))
        {
            fprintf(stderr, "ERROR: HabitatSpace >>>> readGeometry >>>> cell numbers are out of sequence in file %s\n", cellDataFile);
            fflush(0);
            exit(1);
        }

        if(transect == oldtransect+1) 
        {
            oldtransect = transect;   
            prevupstream = oldupstream;
            oldupstream = upstream;
            prevlefty = 0.0;
        }

        if(checkTransect)
        {
            checkTransect = NO;
            if((transect == 1) && (prevTransect == 0))
            {
               prevTransect = transect;
            }
            else if(transect == (prevTransect + 1))
            {
               prevTransect = transect;
            } 
            else if(transect != (prevTransect + 1))
            {
               fprintf(stderr, "ERROR: HabitatSpace >>>> readGeometry >>>> transects are out of sequence in file %s\n", cellDataFile);
               fflush(0);
               exit(1);
            }
        }
        if(prevTransect != transect)
        {
            checkTransect = YES;
        }


        xExtent = upstream - prevupstream;
        yExtent = lefty - prevlefty;

        cell = [Cell createBegin: [self getZone]];

        [cell setSpace: self];
        [cell setModel: model];
        [cell setReach: self];
     
        [cell setTimeManager: timeManager];

        [cell setGenerator: randGen];
       
        [cell setRasterResolution: rasterResolution];
        [cell setRasterResolutionX: rasterResolutionX];
        [cell setRasterResolutionY: rasterResolutionY];
        [cell setRasterColorVariable: rasterColorVariable];

        [cell setTransect: transect];
        [cell setCellNo: cellno];

        [cell setRefPoint:  [[[Vector create: [cell getZone]] 
                   setElement: XCOORDINATE withValue: 100*upstream] 
                   setElement: YCOORDINATE withValue: 100*lefty ]];

       midPointX = 100*(upstream + prevupstream)/2.0;
       midPointY = 100*(lefty + prevlefty)/2.0; 

       [cell setMidPoint: 
             [[[Vector 
                create: [cell getZone]]
                setElement: XCOORDINATE withValue: midPointX]
                setElement: YCOORDINATE withValue: midPointY]];
                //setElement: XCOORDINATE withValue: 100*(upstream + prevupstream)/2.0]
                //setElement: YCOORDINATE withValue: 100*(lefty + prevlefty)/2.0 ]];

      [cell setExtent: 
                [[[Vector
                    create: [cell getZone]]
                   setElement: XCOORDINATE withValue: 100*xExtent]
                   setElement: YCOORDINATE withValue: 100*yExtent]];

        [cell setCellFracShelter: fracshel]; 
        [cell setCellFracSpawn: fracspawn];
        [cell setDistanceToHide: 100*disttohi]; 

        [cell setFishParamsMap: fishParamsMap];
        [cell setNumberOfSpecies: numberOfSpecies];

        [cell setHabShearParamA: habShearParamA
                 habShearParamB: habShearParamB];

        [cell setHabShelterSpeedFrac: habShelterSpeedFrac];

        cell = [cell createEnd];

        [self putObject: cell atX: transect-1 Y: cellno-1];

        [listOfCells addLast: cell];
        
        prevlefty = lefty;

        streamLength = upstream*100;  //streamLength will be the last upstream x

        reachLength = (double) streamLength;

        prevCellno = cellno;

      } 

  }

  listOfCellsNdx = [listOfCells listBegin: habitatZone];


  //cleanup

  [[self getZone] free: header1];
  fclose (geomFP);
 
  return self;
}


/////////////////////////////////////////////////////////////////////////////////
//
// calcSpaceVariables
//
/////////////////////////////////////////////////////////////////////////////////
- calcSpaceVariables 
{
  id aCell=nil;
  //id prevCell=nil;
  //id nextCell=nil;
  id cellNdx=nil;
  id xCellNdx=nil;
  double myX=0.0, myY=0.0;
  double  myMidPointX,myMidPointY;
  double maxX=0.0, maxY=0.0;
  int ndx;
  //int myTransect = 0, prevTransect;
  Vector ** boundary;

  double minY=LARGEINT;

  cellNdx = [listOfCells listBegin: scratchZone];
  xCellNdx = [listOfCells listBegin: scratchZone];

  while (([cellNdx getLoc] != End) && ((aCell = [cellNdx next]) != nil))
  {
    //
    //Allocate boundary vectors for each cell.  We need twice
    //the number of DIMENSIONs 
    //

    boundary = (Vector **) [ZoneAllocMapper allocBlockIn: habitatZone 
                                                  ofSize: (size_t) (2*DIMENSION)*sizeof(Vector*)];

      for (ndx=0;ndx<2*DIMENSION;ndx++)
      {
	boundary[ndx]= [Vector create: [self getZone]];
      }

     myX = [[aCell getRefPoint] getElement: XCOORDINATE];
     myY = [[aCell getRefPoint] getElement: YCOORDINATE];
     myMidPointX = [[aCell getMidPoint] getElement: XCOORDINATE];
     myMidPointY = [[aCell getMidPoint] getElement: YCOORDINATE];
     

     [boundary[0] setElement: XCOORDINATE withValue: 2*myMidPointX - myX];
     [boundary[0] setElement: YCOORDINATE withValue: 2*myMidPointY - myY];

     [boundary[1] setElement: XCOORDINATE withValue: myX];
     [boundary[1] setElement: YCOORDINATE withValue: 2*myMidPointY - myY];

     [boundary[2] setElement: XCOORDINATE withValue: 2*myMidPointX - myX];
     [boundary[2] setElement: YCOORDINATE withValue: myY];

     [boundary[3] setElement: XCOORDINATE withValue: myX];
     [boundary[3] setElement: YCOORDINATE withValue: myY];

     [aCell setBoundary: boundary];
     [aCell calcCellShelterArea];

     maxX = (maxX > myX ) ? maxX : myX;
     maxY = (maxY > myY ) ? maxY : myY;
  
     minY = (minY < myY) ? minY : myY;

     maxCellY = maxY; // used to draw the barriers
     minCellY = minY; 
       


  } // while

  [cellNdx setLoc: Start];
  while (([cellNdx getLoc] != End) && ((aCell = [cellNdx next]) != nil))
  {
      [self getAdjacentCells: aCell];
  }



  [cellNdx drop];
  [xCellNdx drop];


  pixelsX = (unsigned int)maxX;  // double to int
  pixelsY = (unsigned int)maxY;


  return self;
}



////////////////////////////////////////////////
//
// getHabSearchProd
//
////////////////////////////////////////////////
- (double) getHabSearchProd {
  return habSearchProd;
}




/////////////////////////////////////////////////////////
//
// getHabDriftConc
//
/////////////////////////////////////////////////////////
- (double) getHabDriftConc {
  return habDriftConc;
}

///////////////////////////////////////////////////
//
// getHabDriftRegenDist
//
/////////////////////////////////////////////////
- (double) getHabDriftRegenDist {
   return habDriftRegenDist;
}

//////////////////////////////////////////////////////////
//
// getHabPreyEnergyDensity
//
/////////////////////////////////////////////////////////
- (double) getHabPreyEnergyDensity {
  return habPreyEnergyDensity;
}




////////////////////////////////////////////////
//
// getDayLength
//
////////////////////////////////////////////////
- (double) getDayLength {
  return dayLength;
}


///////////////////////////////////////////////////////////////////////////
//
// probeCellAtX
//
//////////////////////////////////////////////////////////////////////////
#import <simtoolsgui.h>
- probeCellAtX: (int) probedX Y: (int) probedY {
  id cell=nil;
  cell = [self _getCellContainingX: probedX Y: probedY];
  // throw up a probe for that cell
  if( cell != nil)
  CREATE_ARCHIVED_PROBE_DISPLAY (cell);
  return self;
}



/////////////////////////////////////////////////////////////////////////////
//
// probeFishAtX
//
////////////////////////////////////////////////////////////////////////////
- probeFishAtX: (int) probedX Y: (int) probedY {
  id cell=nil;
  id fish=nil;
  id redd=nil;
  id warnFish=nil;
  //unsigned fishX, fishY;
  id <ListIndex> fishNdx=nil;
  id <ListIndex> reddNdx=nil;
  //id <List> tempList;

  // get the cell
  if ((cell = [self _getCellContainingX: probedX Y: probedY]) == nil)
    return nil;

  fishNdx = [[cell getFishIContain] listBegin: scratchZone];
  while (([fishNdx getLoc] != End) &&
         ((fish = [fishNdx next]) != nil)) {

   warnFish = fish;

    // At this time this will create a a probe display for each fish in the Cell
    CREATE_PROBE_DISPLAY(fish);
   
  }

  //if(warnFish == nil)
    //[WarningMessage raiseEvent: 
        //"Couldn't match probed (%d,%d) with a fish.",probedX,probedY];

  [fishNdx drop];





  reddNdx = [[cell getReddsYouContain] listBegin: scratchZone];
  while (([reddNdx getLoc] != End) &&
         ((redd = [reddNdx next]) != nil)) {

    // At this time this will create a a probe display for each redd in the Cell
    CREATE_PROBE_DISPLAY(redd);
   
  }


   [reddNdx drop];


  return self;
}



/////////////////////////////////////////////////////////////////////////////
//
// getCellContainingX
//
/////////////////////////////////////////////////////////////////////////////
- _getCellContainingX: (int) probedX Y: (int) probedY 
{
  Cell* transectCell0 = nil;
  Cell* cellOnTransect = nil;
  int xNdx=LARGEINT;
  int yNdx=LARGEINT;
  double thisProbedX = 0.0;
  double thisProbedY = 0.0;

  //
  // convert from raster coordinates
  //
  thisProbedX = (double)probedX * rasterResolutionX;
  thisProbedY = (double)probedY * rasterResolutionY;

  fprintf(stdout, "HabitatSpace >>>> getCellContaining >>>> thisProbedX = %f\n", thisProbedX);
  fprintf(stdout, "HabitatSpace >>>> getCellContaining >>>> thisProbedY = %f\n", thisProbedY);
  fflush(0);

  //
  // determine what cell covers that (x,y) point
  //  first find the right transect
  //
  xNdx = 0L;
  while((xNdx < xsize) && 
          (((transectCell0 = [self getObjectAtX: xNdx Y: 0L]) == nil) 
            || ([transectCell0 containsX: (double) thisProbedX] == NO))) 
  {

    xNdx++;
  }

  if((transectCell0 == nil) || (xNdx >= xsize) || (xNdx == LARGEINT))
  {
    return nil;
  }

  //
  //  then find the right cell within the transect
  //
  yNdx = 0L;
  while((yNdx < ysize) && 
          (((cellOnTransect = [self getObjectAtX: xNdx Y: yNdx]) == nil) ||
           ([cellOnTransect containsY: (double) thisProbedY] == NO) )) 
  {
    yNdx++;
  }

  if((cellOnTransect == nil) || (yNdx >= ysize) || (yNdx == LARGEINT))
  {
    return nil;
  }

  return cellOnTransect;

}



///////////////////////////////////////////////////////////////////////
//
// getPixelsX
//
//////////////////////////////////////////////////////////////////////
- (unsigned) getPixelsX {
  return pixelsX;
}


//////////////////////////////////////////////////////////////////////
//
// getPixelsY
//
//////////////////////////////////////////////////////////////////////
- (unsigned) getPixelsY {
  return pixelsY;
}




////////////////////////////////////////////////////////////////////////
//
// getNeighborsWithin
//
// Comment: List of neighbors does not include self
//
///////////////////////////////////////////////////////////////////////
- (id <List>) getNeighborsWithin: (double) aRange 
                              of: refCell 
                        withList: (id <List>) aCellList
{
  int ndx;
  id <ListIndex> cellNdx;
  id tempCell;
  id <List> listOfCellsWithinRange = aCellList;
  id <List> adjacentCells = [refCell getListOfAdjacentCells];
  int adjacentCellCount;
  Vector * midPoint=nil;
  //Vector * diff=nil;
  Vector * tempVector=nil;

  cellNdx = [listOfCells listBegin: scratchZone];

  //fprintf(stdout, "HabitatSpace >>>> getNeigborsWithin >>>> BEGIN\n");
  //fflush(0);


  if(listOfCellsWithinRange == nil)
  {
      fprintf(stderr, "ERROR: HabitatSpace >>>> getNeighborsWithin >>>> listOfCellsWithinRange is nil\n");
      fflush(0);
      exit(1);
  }
  if([listOfCellsWithinRange getCount] != 0)
  {
      // 
      // The list from the fish must be empty
      //
      fprintf(stderr, "ERROR: HabitatSpace >>>> getNeighborsWithin >>>> listOfCellsWithinRange is not empty\n");
      fflush(0);
      exit(1);
  }
  if(adjacentCells == nil)
  {
      fprintf(stderr, "ERROR: HabitatSpace >>>> getNeighborsWithin >>>> adjacentCells is nil\n");
      fflush(0);
      exit(1);
  }

  adjacentCellCount = [adjacentCells getCount];

  if(adjacentCellCount == 0)
  {
      // 
      // The list of adjacent cells shouldn't be empty
      //
      fprintf(stderr, "ERROR: HabitatSpace >>>> getNeighborsWithin >>>> adjacentCells is empty\n");
      fflush(0);
      exit(1);
  }

  midPoint = [refCell getMidPoint];

  tempVector = [Vector create: scratchZone]; //Create one Vector
  while(([cellNdx getLoc] != End) && ((tempCell = [cellNdx next]) != nil)) 
  {
    if(refCell == tempCell) continue;


    for (ndx=0;ndx<DIMENSION;ndx++) 
    {
        [tempVector setElement: ndx 
                     withValue: [midPoint getElement: ndx] - [[tempCell getMidPoint] getElement: ndx]];
    }

    if ([tempVector getMagnitude] <= aRange) 
    {
        [listOfCellsWithinRange addLast: tempCell];
    }

  }

        
  //
  // Now, ensure listOfCellsWithinRange contains refCell's
  // adjacentCells
  //
  {
     int i;
     for(i = 0; i < adjacentCellCount; i++)
     {
         Cell* adjacentCell = [adjacentCells atOffset: i]; 
         if([listOfCellsWithinRange contains: adjacentCell] == NO)
         {
            [listOfCellsWithinRange addLast: adjacentCell];
         }
     } 
  }

  // cleanup
  [tempVector drop];
  [cellNdx drop];

  //
  // ADJACENT REACHES
  //
  //
  if(aRange > [midPoint getElement: XCOORDINATE])
  {
       int i;
       id oRCell;
       double oRRange = aRange - [midPoint getElement: XCOORDINATE];
 
       // if there are reaches whose downstream ends are links to the current reach's
       // downstream end habDownstreamLinksToDS is not empty then a message is sent to each reach on
       // habDownstreamLinksToDS to obtain a list of cellswithin a specified distance 
       // of its downstream end. This distance is equal to aRange - the X coordinate
       // of refCell's midPoint.
       // the cells are added to the end of listOfCellsWithinRange

       //fprintf(stdout, "HabitatSpace >>>> getNeighborsWithin \n");
       //fprintf(stdout, "HabitatSpace >>>> habDownstreamLinksToDS\n");
       //xprint(habDownstreamLinksToDS);
       //fflush(0);

        //if(oRRange > 0.0)
        //{
            for(i = 0; i < [habDownstreamLinksToDS getCount]; i++)
            {
                id <List> oRCellList;
                id <ListIndex> oRCellNdx;
                id anotherReach = [habDownstreamLinksToDS atOffset: i];
 
                oRCellList = [anotherReach getDownstreamCellsWithinRange: oRRange];


                //xprint(oRCellList);

                if([oRCellList getCount] > 0)
                {
                    oRCellNdx = [oRCellList listBegin: scratchZone];
    
                    while(([oRCellNdx getLoc] != End) && ((oRCell = [oRCellNdx next]) != nil))
                    {
                       [listOfCellsWithinRange addLast: oRCell];
                    }
           
                    [oRCellNdx drop];

               }

            }



       // If there area reaches whoes upstream ends are linked to the current reach's 
       // downstream end (habDownstreamLinksToUS is not empty), then a message is
       // sent to each reach on habDownstreamLinksToUS to obtain a list of cells
       // within a distance of its upstream end. this distance is equal to aRange minus
       // the X coordinate of refCell's midpoint. These cells are added to the end of
       // listOfCellsWithinRange.
       //fprintf(stdout, "HabitatSpace >>>> getNeighborsWithin \n");
       //fprintf(stdout, "HabitatSpace >>>> habDownstreamLinksToUS\n");
       //xprint(habDownstreamLinksToUS);
       //fflush(0);

            for(i = 0; i < [habDownstreamLinksToUS getCount]; i++)
            {
                id <List> oRCellList;
                id <ListIndex> oRCellNdx;
                id anotherReach = [habDownstreamLinksToUS atOffset: i];
    
                oRCellList = [anotherReach getUpstreamCellsWithinRange: oRRange];

                //xprint(oRCellList);

                if([oRCellList getCount] > 0)
                {
                    oRCellNdx = [oRCellList listBegin: scratchZone];
    
                    while(([oRCellNdx getLoc] != End) && ((oRCell = [oRCellNdx next])!= nil))
                    {
                       [listOfCellsWithinRange addLast: oRCell];
                    }
               
                    [oRCellNdx drop];
                }

            }
 
      //}


  }
  if((aRange + [[refCell getMidPoint] getElement: XCOORDINATE]) > reachLength)
  { 
       int i;
       id oRCell;
       double oRRange = (aRange + [midPoint getElement: XCOORDINATE]) - reachLength;


       // If there area reaches whose downstream end are linked to the current
       // reach's upstream end ( habUpstreamLinksToDS is not empty), then a message
       // sent to each reach on habStreamLinksToDS to obtain a list of cells within
       // distance of its downtstream end. This distance is equal to aRange + refCell's
       // midpoint, minus the current reaches length in the X dimension. The cells returned
       // in response to this message are added to the end of listOfCellsWihinRange.

       //fprintf(stdout, "HabitatSpace >>>> getNeighborsWithin \n");
       //fprintf(stdout, "HabitatSpace >>>> habUpstreamLinksToDS\n");
       //xprint(habUpstreamLinksToDS);
       //fflush(0);
         
        //if((oRRange > 0.0) && (oRRange < reachLength))
        //{
	    for(i = 0; i < [habUpstreamLinksToDS getCount]; i++)
	    {
	        id <List> oRCellList;
	        id <ListIndex> oRCellNdx;
	        id anotherReach = [habUpstreamLinksToDS atOffset: i];
 
                oRCellList = [anotherReach getDownstreamCellsWithinRange: oRRange];


                //xprint(oRCellList);


	        if([oRCellList getCount] > 0)
	        {
		    oRCellNdx = [oRCellList listBegin: scratchZone];

		    while(([oRCellNdx getLoc] != End) && ((oRCell = [oRCellNdx next]) != nil))
		    {
		       [listOfCellsWithinRange addLast: oRCell];
		    }
	       
		    [oRCellNdx drop];

	       }

	    }



  
 
       // If there are reaches whose upstream ends are linked to the current reach's upstream
       // end (habUpstreamLinksToUS is not empty), then a message is sent to each reach on 
       // habUpstreamLinksToUS to obtain a list of cells within a Distance of its up stream
       // end. This distance is equal to a aRange + refCell's midpoint, minus the current
       // reach's total length in the X dimension. The returned cells are added to the end of 
       // listOfCellsWithinRange.

       //fprintf(stdout, "HabitatSpace >>>> getNeighborsWithin \n");
       //fprintf(stdout, "HabitatSpace >>>> habUpstreamLinksToUS\n");
       //xprint(habUpstreamLinksToUS);
       //fflush(0);
            
	    for(i = 0; i < [habUpstreamLinksToUS getCount]; i++)
	    {
	        id <List> oRCellList;
	        id <ListIndex> oRCellNdx;
	        id anotherReach = [habUpstreamLinksToUS atOffset: i];
 
	        oRCellList = [anotherReach getUpstreamCellsWithinRange: oRRange];

                xprint(oRCellList);

	        if([oRCellList getCount] > 0)
	        {
		    oRCellNdx = [oRCellList listBegin: scratchZone];

		    while(([oRCellNdx getLoc] != End) && ((oRCell = [oRCellNdx next]) != nil))
		    {
		       [listOfCellsWithinRange addLast: oRCell];
		    }
	   
		    [oRCellNdx drop];

	       }

	    }
  
          //}

  }

  //fprintf(stdout, "HabitatSpace >>>> getNeigborsWithin >>>> END\n");
  //fflush(0);

  return listOfCellsWithinRange;
}



- (id <List>) getDownstreamCellsWithinRange: (double) aRange
{
    id cell = nil;

    [listOfCellsNdx setLoc: Start];

    [tempCellList removeAll];

    while(([listOfCellsNdx getLoc] != End) && ((cell = [listOfCellsNdx next]) != nil))
    { 
        if([[cell getMidPoint] getElement: XCOORDINATE] < aRange)
        {
             [tempCellList addLast: cell]; 	 
        }
    
    }


    return tempCellList;

}


- (id <List>) getUpstreamCellsWithinRange: (double) aRange
{
    id cell = nil;

    [listOfCellsNdx setLoc: Start];

    [tempCellList removeAll];

    while(([listOfCellsNdx getLoc] != End) && ((cell = [listOfCellsNdx next]) != nil))
    { 
        if([[cell getMidPoint] getElement: XCOORDINATE] > (reachLength - aRange))
        {
             [tempCellList addLast: cell]; 	 
        }
    
    }


    return tempCellList;

}
     

///////////////////////////////////////
//
// getReachLength
//
//////////////////////////////////////
- (double) getReachLength
{
    return reachLength;
}




//////////////////////////////////////////////////////////
//
// getAdjacentCells
//
/////////////////////////////////////////////////////////
- getAdjacentCells: refCell 
{
    double adjacentX,adjacentY;
    id <List> listOfAdjacentCells = nil;
    id cell;

    //fprintf(stdout, "HabitatSpace >>>> getAdjacentCells >>>> BEGIN\n");
    //fflush(0);

    listOfAdjacentCells = [refCell getListOfAdjacentCells];

    if(listOfAdjacentCells == nil)
    {
         fprintf(stderr, "ERROR: HabitatSpace >>>>> getAdjacentCells >>>> listOfAdjacentCells is nil\n");
         fflush(0);
         exit(1);
    }

    adjacentX = [[refCell getMidPoint] getElement: XCOORDINATE];
    adjacentY = [[refCell getBoundary][0] getElement: YCOORDINATE] - 1;

    if((cell = [self _getCellContainingFloatX: adjacentX floatY: adjacentY]) != nil) 
    {
        [listOfAdjacentCells addLast: cell];
    }


    adjacentX = [[refCell getMidPoint] getElement: XCOORDINATE];
    adjacentY = [[refCell getBoundary][2] getElement: YCOORDINATE] + 1;

    if((cell = [self _getCellContainingFloatX: adjacentX floatY: adjacentY]) != nil) 
    {
      [listOfAdjacentCells addLast: cell];
    }
    
    adjacentX = [[refCell getBoundary][0] getElement: XCOORDINATE] - 1;
    adjacentY = [[refCell getMidPoint] getElement: YCOORDINATE];

    if((cell = [self _getCellContainingFloatX: adjacentX floatY: adjacentY]) != nil) 
    {
      [listOfAdjacentCells addLast: cell];
    }


    adjacentX = [[refCell getBoundary][1] getElement: XCOORDINATE] + 1;
    adjacentY = [[refCell getMidPoint] getElement: YCOORDINATE];

    if( (cell = [self _getCellContainingFloatX: adjacentX floatY: adjacentY]) != nil)
    {
      [listOfAdjacentCells addLast: cell];
    }

    //fprintf(stdout, "HabitatSpace >>>> getAdjacentCells >>>> END\n");
    //fflush(0);

    return listOfAdjacentCells;
}


/////////////////////////////////////////////////////////////////////////////
//
// getCellContainingFloatX
//
/////////////////////////////////////////////////////////////////////////////
- _getCellContainingFloatX: (double) probedX floatY: (double) probedY 
{
  Cell* transectCell0=nil;
  Cell* cellOnTransect=nil;
  int xNdx=LARGEINT;
  int yNdx=LARGEINT;
  double thisProbedX = probedX;
  double thisProbedY = probedY;

  //
  // Determine what cell covers that (x,y) point
  // first find the right transect
  //

  //
  // I modified this and the cell methods to include containsDoubleX containsDoubleY
  // 2/5/04
  //


  xNdx = 0L;
  while((xNdx < xsize) && 
          (((transectCell0 = [self getObjectAtX: xNdx Y: 0L]) == nil) 
            || ([transectCell0 containsDoubleX: (double) thisProbedX] == NO))) 
  {

      xNdx++;
  }

  if((transectCell0 == nil) || (xNdx >= xsize) || (xNdx == LARGEINT)) 
  {
      return nil;
  }
  
  //
  // Then find the right cell within the transect
  //
  yNdx = 0L;
  while((yNdx < ysize) && 
          (((cellOnTransect = [self getObjectAtX: xNdx Y: yNdx]) == nil) ||
           ([cellOnTransect containsDoubleY: (double) thisProbedY] == NO))) 
  {
     yNdx++;
  }

  if((cellOnTransect == nil) || (yNdx >= ysize) || (yNdx == LARGEINT))
  {
      return nil;
  }

  return cellOnTransect;
}


////////////////////////////////////////////////////////
//
// readTomorrowsFlow
//
///////////////////////////////////////////////////////
- (double) readTomorrowsFlow : (time_t) aModelTime_t 
{ 
  double aRiverFlow=-LARGEINT;
  time_t tomorrowsTime;
  tomorrowsTime = aModelTime_t + 86400;
  aRiverFlow = [flowInputManager getValueForTime: tomorrowsTime];
  return aRiverFlow;
}



//////////////////////////////////////// 
//
// getModelTime
//
////////////////////////////////////////
-(time_t) getModelTime 
{
   return modelTime_t;
}


///////////////////////////////////////////////////////////////////////
//
// updateHabitat
//
//////////////////////////////////////////////////////////////////////
-   updateHabitatWithTime: (time_t) aModelTime_t
    andWithModelStartFlag: (BOOL) aStartFlag
{
  //fprintf(stdout, "HABITATSPACE >>>> updateHabitat >>>> BEGIN\n");
  //fflush(0);
   
  modelTime_t = aModelTime_t;

  sprintf(modelDate, "%s", [timeManager getDateWithTimeT: modelTime_t]);

  //modelDate = [timeManager getDateWithTimeT: modelTime_t];

  strncpy(Date, modelDate, strlen(modelDate) + 1);

  if(aStartFlag == NO)
  { 
      yesterdaysRiverFlow = riverFlow;
      riverFlow = [flowInputManager getValueForTime: modelTime_t];
      tomorrowsRiverFlow = [flowInputManager getValueForTime: (modelTime_t + 86400)];
  }      
  else // aStartFlag == YES
  {
      //
      // On the first day yesterdaysRiverFlow is set to todays riverFlow 
      //
      riverFlow = [flowInputManager getValueForTime: aModelTime_t];
      yesterdaysRiverFlow = riverFlow;
      tomorrowsRiverFlow = [flowInputManager getValueForTime: (modelTime_t + 86400)];
  }
       
  dayLength = [solarManager getDayLength];
  [self updateCellHydraulicsWith: riverFlow];
  [self calcWettedArea];

  temperature = [temperatureInputManager getValueForTime: aModelTime_t];
  turbidity = [turbidityInputManager getValueForTime: aModelTime_t];

  [self updateFlowChange];

  numPiscivorousFish = 0;

  [listOfCells forEach: M(updateHabitatSurvivalProb)];
  [listOfCells forEach: M(updateDSCellHourlyTotal)];
  [listOfCells forEach: M(resetAvailHourlyTotal)];
  [listOfCells forEach: M(resetShelterAreaAvailable)];


  //fprintf(stderr, "HABITATSPACE >>>> updateHabitat >>>> END\n");
  //fflush(0);

  return self;
}


/////////////////////////////////////////////////////////////////////////
//
// updateCellHydraulicsWith
//
// Set the depth and velocities in each cell
//
///////////////////////////////////////////////////////////////////////
- updateCellHydraulicsWith: (double) aFlow
{
  id <ListIndex> cellNdx;
  id cell;
  int interpolationIndex = [flowInterpolator getTableIndexFor: aFlow];
  double interpFraction = [flowInterpolator getInterpFractionFor: aFlow];

  cellNdx = [listOfCells listBegin: scratchZone];

  while (([cellNdx getLoc] != End) && ((cell = [cellNdx next]) != nil)) 
  {
     [cell calcDepthAndVelocityWithIndex: interpolationIndex
                      withInterpFraction: interpFraction];

  }

  [cellNdx drop];

  return self;
}

////////////////////////////////////////////////
//
// calcWettedArea
//
////////////////////////////////////////////////
- calcWettedArea
{
  id aCell = nil;

  habWettedArea = 0.0;

  [listOfCellsNdx setLoc: Start];
    
   while(([listOfCellsNdx getLoc] != End)
            && ((aCell = [listOfCellsNdx next]) != nil))
  {
      if([aCell getDepth] > 0.0)
      {
         habWettedArea += [aCell getArea];
      }
  }

  return self;
}


////////////////////////////////////////////////////////////////
//
// getTemperature
//
///////////////////////////////////////////////////////////////
- (double) getTemperature 
{
    return temperature;
}


////////////////////////////////////////////////////////////////
//
// getTurbidity
//
///////////////////////////////////////////////////////////////
- (double) getTurbidity 
{
   return turbidity;
}




/////////////////////////////////////////////////////////////
//
// updateFlowChange
//
// Return the absolute value of the change
// flow from yesterday
//
// NOTE: They are NOT logarithms
//
//////////////////////////////////////////////////////////
- updateFlowChange 
{
  double diff;
  diff = yesterdaysRiverFlow - riverFlow;
  flowChange = sqrt(pow(diff,2));
  return self;
}


///////////////////////////////////////////////////////////
//
// getFlowChange
//
//////////////////////////////////////////////////////////
- (double) getFlowChange
{
  return flowChange;
}


//////////////////////////////////////////////////////////////
//
// getYesterdaysRiverFlow
//
///////////////////////////////////////////////////////////
- (double) getYesterdaysRiverFlow 
{
 return yesterdaysRiverFlow;
}


//////////////////////////////////////////////////////////////
//
// getRiverFlow
//
///////////////////////////////////////////////////////////
- (double) getRiverFlow 
{
 return riverFlow;
}


//////////////////////////////////////////////////////////////
//
// getTomorrowsRiverFlow
//
// pass through from cell
//
//
///////////////////////////////////////////////////////////
- (double) getTomorrowsRiverFlow 
{
 return tomorrowsRiverFlow;
}


//////////////////////////////////////////////////
//
// getHabMaxSpawnFlow
//
//////////////////////////////////////////////////
- (double) getHabMaxSpawnFlow
{
    return habMaxSpawnFlow;
}

////////////////////////////////////////////////////////////////////
//
// buildCells
//
///////////////////////////////////////////////////////////////////
- buildCells 
{

  fprintf(stderr,"HabitatSpace >>>>> buildCells >>>> BEGIN\n");
  fflush(0);

  [self readGeometry];  // and create the cells
  [self calcSpaceVariables];

  fprintf(stderr,"HabitatSpace >>>>> buildCells >>>> BEGIN\n");
  fflush(0);


  return self;
}

/////////////////////////////////////////////
//
// setModelStartTime:andEndTime
//
/////////////////////////////////////////////
- setModelStartTime: (time_t) startTime  
         andEndTime: (time_t) endTime 
{
  modelStartTime = startTime;
  modelEndTime = endTime;
  return self;
}


////////////////////////////////////////
//
// setDataStartTime: andEndTime
//
////////////////////////////////////////
- setDataStartTime: (time_t) aDataStartTime
        andEndTime: (time_t) aDataEndTime
{
   dataStartTime = aDataStartTime;
   dataEndTime = aDataEndTime;
   return self;
}






- printCellDepthReport 
{
 
  FILE * reportPtr=NULL;

  BOOL loopFirstTime=YES;

  char * printString=(char *) NULL;
  char * print1String=(char *) NULL;
  char * print2String=(char *) NULL;

  id <ListIndex> cellNdx;
  id nextCell;
  int currTransect=1;
  int transect;
  int cellNo;
  double myRiverFlow;
  double depth;
  char * cellNo1String=(char *) NULL;
  char * cellNo2String=(char *) NULL;
  char * cellPrintString=(char *) NULL;


  



  if(depthReportFirstWrite == YES) 
  {
      if((reportPtr = fopen(cellDepthReportFile,"w+")) == NULL) 
      {
           fprintf(stderr, "ERROR: HabitatSpace >>>> printCellDepthReport  >>>> Cannot open file %s",cellDepthReportFile);
           fflush(0);
           exit(1);
      }

      fprintf(reportPtr, "%-6s%-12s%-12s\n","TSEC", "RiverFlow", "CellDepths:");
      fflush(reportPtr);
  }


  if(depthReportFirstWrite == NO) 
  {
    if((reportPtr = fopen(cellDepthReportFile,"a")) == NULL)
    {
        fprintf(stderr, "ERROR: HabitatSpace >>>> printCellDepthReport  >>>> Cannot open file %s",cellDepthReportFile);
        fflush(0);
        exit(1);
    }
  }

  print1String = (char *) [[self getZone] alloc: 300*sizeof(char)];
  print2String = (char *) [[self getZone] alloc: 300*sizeof(char)];

  cellNo1String = (char *) [[self getZone] alloc: 300*sizeof(char)];
  cellNo2String = (char *) [[self getZone] alloc: 300*sizeof(char)];

  cellNdx = [listOfCells listBegin: [self getZone]];
 
  //fprintf(reportPtr,"\nThe number of cells = %d \n", [listOfCells getCount]);

  while(([cellNdx getLoc] != End) && ( (nextCell = [cellNdx next]) != nil)) 
  {
     transect = [nextCell getTransect];
     cellNo   = [nextCell getCellNo];
     myRiverFlow = [nextCell getRiverFlow];
     depth    = [nextCell getDepth];

    if(loopFirstTime == YES) 
    {
       sprintf(cellNo1String,"%-18c",' ');
       sprintf(print1String,"%-6d%-12f", transect, myRiverFlow);
       loopFirstTime = NO;
    }

    if(currTransect == transect) 
    {
        sprintf(cellNo2String,"%-12d",cellNo);
        sprintf(print2String,"%-12f", depth);
        cellPrintString = strcat(cellNo1String,cellNo2String);
        printString = strcat(print1String, print2String);
        continue;
    }

    //fprintf(reportPtr,"%s\n",cellPrintString);

    fprintf(reportPtr,"%s\n",printString);

    currTransect = transect;

    sprintf(print1String,"%-6d%-12f", transect,myRiverFlow);
    sprintf(print2String,"%-12f", depth);
    sprintf(cellNo1String,"%-18c",' ');
    sprintf(cellNo2String,"%-12d",cellNo);
    cellPrintString = strcat(cellNo1String, cellNo2String);
    printString = strcat(print1String, print2String);

 } //while

      
  //fprintf(reportPtr,"%s\n",cellPrintString);

  fprintf(reportPtr,"%s\n",printString);
  fflush(0);
  
  [[self getZone] free: print1String];
  [[self getZone] free: print2String];
  [[self getZone] free: cellNo1String];
  [[self getZone] free: cellNo2String];

  [cellNdx drop];
  fclose(reportPtr);

  depthReportFirstWrite = NO;

  return self;
}


- printCellVelocityReport 
{
 
  FILE * reportPtr=NULL;
  //const char *fileName = "Cell_Flow_Velocity_Test.rpt";

  //static BOOL velocityReportFirstWrite = YES;
  BOOL loopFirstTime=YES;

  char * printString=(char *) NULL;
  char * print1String=(char *) NULL;
  char * print2String=(char *) NULL;

  id <ListIndex> cellNdx;
  id nextCell;
  int currTransect=1;
  int transect;
  int cellNo;
  double myRiverFlow;
  double velocity;
  char * cellNo1String=(char *) NULL;
  char * cellNo2String=(char *) NULL;
  char * cellPrintString=(char *) NULL;

  if(velocityReportFirstWrite == YES) {
    if( (reportPtr = fopen(cellVelocityReportFile,"w+")) == NULL ) {
      [InternalError raiseEvent:"ERROR: Cannot open file %s",cellVelocityReportFile];
    }
  fprintf(reportPtr,"%-6s%-12s%-12s\n","TSEC", "RiverFlow", "CellVelocities:");
  }


  if(velocityReportFirstWrite == NO) {
    if( (reportPtr = fopen(cellVelocityReportFile,"a")) == NULL )
    [InternalError raiseEvent:"ERROR: Cannot open file %s",cellVelocityReportFile];
  }

  print1String = (char *) [[self getZone] alloc: 300*sizeof(char)];
  print2String = (char *) [[self getZone] alloc: 300*sizeof(char)];

  cellNo1String = (char *) [[self getZone] alloc: 300*sizeof(char)];
  cellNo2String = (char *) [[self getZone] alloc: 300*sizeof(char)];

  cellNdx = [listOfCells listBegin: [self getZone]];
 
  //fprintf(reportPtr,"\nThe number of cells = %d \n", [listOfCells getCount]);

  while( ([cellNdx getLoc] != End) && ( (nextCell = [cellNdx next]) != nil) ) {
  

       transect = [nextCell getTransect];
       cellNo   = [nextCell getCellNo];
       myRiverFlow = [nextCell getRiverFlow];
       velocity    = [nextCell getCellVelocity];

   if(loopFirstTime == YES) {
      sprintf(cellNo1String,"%-18c",' ');
      sprintf(print1String,"%-6d%-12f", transect, myRiverFlow);
      loopFirstTime = NO;
   }

   if(currTransect == transect) {

       sprintf(cellNo2String,"%-12d",cellNo);
       sprintf(print2String,"%-12f", velocity);
       cellPrintString = strcat(cellNo1String,cellNo2String);
       printString = strcat(print1String, print2String);
       continue;
   }

  fprintf(reportPtr,"%s\n",cellPrintString);

  fprintf(reportPtr,"%s\n",printString);

  currTransect = transect;

  sprintf(print1String,"%-6d%-12f", transect,myRiverFlow);
  sprintf(print2String,"%-12f", velocity);
  sprintf(cellNo1String,"%-18c",' ');
  sprintf(cellNo2String,"%-12d",cellNo);
  cellPrintString = strcat(cellNo1String, cellNo2String);
  printString = strcat(print1String, print2String);

 }

      
fprintf(reportPtr,"%s\n",cellPrintString);

fprintf(reportPtr,"%s\n",printString);
fflush(0);

[[self getZone] free: print1String];
[[self getZone] free: print2String];
[[self getZone] free: cellNo1String];
[[self getZone] free: cellNo2String];


[cellNdx drop];
fclose(reportPtr);

velocityReportFirstWrite = NO;

return self;
}


///////////////////////////////////////
//
// buildCellFishInfoReporter
//
//////////////////////////////////////
- buildCellFishInfoReporter
{
  BOOL fileOverWrite = TRUE;
  BOOL suppressBreakoutColumns = NO;


  //fprintf(stdout, "HabitatSpace >>>> buildCellFishInfoReporter >>>> BEGIN\n");
  //fflush(0);
 
  if(speciesSymbolList == nil)
  {
     fprintf(stderr, "ERROR: HabitatSpace >>>> buildCellFishInfoReporter >>>> speciesSymbolList is nil\n");
     fflush(0);
     exit(1);
  }

  if(ageSymbolList == nil)
  {
     fprintf(stderr, "ERROR: HabitatSpace >>>> buildCellFishInfoReporter >>>> ageSymbolList is nil\n");
     fflush(0);
     exit(1);
  }


  if((scenario == 0) || (replicate == 0))
  {
     fprintf(stderr, "ERROR: HabitatSpace >>>> buildCellFishInfoReporter >>>> scenario or replicate is 0\n");
     fflush(0);
     exit(1);
  }

  if(appendFiles == TRUE)
  {
     fileOverWrite = FALSE;
  }

  if((scenario != 1) || (replicate != 1))
  {
      suppressBreakoutColumns = YES;
      fileOverWrite = FALSE;
  }

  sprintf(cellFishInfoReportFName, "%s%s", reachName, "CellFishInfo.rpt");
  fprintf(stdout, "HabitatSpace >>>> buildCellFishInfoReporter >>>> cellFishInfoReportFName = %s \n", cellFishInfoReportFName);
  fflush(0);

  cellFishInfoReporter = [BreakoutReporter   createBegin: habitatZone
                                             forList: cellFishList
                                  withOutputFilename: (char *) cellFishInfoReportFName
                                   withFileOverwrite: fileOverWrite
                                     withColumnWidth: 25];



  [cellFishInfoReporter addColumnWithValueOfVariable: "scenario"
                                      fromObject: self
                                        withType: "int"
                                       withLabel: "Scenario"];

  [cellFishInfoReporter addColumnWithValueOfVariable: "replicate"
                                      fromObject: self
                                        withType: "int"
                                       withLabel: "Replicate"];

  [cellFishInfoReporter addColumnWithValueOfVariable: "modelDate"
                                      fromObject: self
                                        withType: "string"
                                       withLabel: "ModelDate"];

  [cellFishInfoReporter addColumnWithValueOfVariable: "habCellTransectNumber"
                                      fromObject: self
                                        withType: "int"
                                       withLabel: "TransectNumber"];

  [cellFishInfoReporter addColumnWithValueOfVariable: "habCellNumber"
                                      fromObject: self
                                        withType: "int"
                                       withLabel: "CellNumber"];

  [cellFishInfoReporter addColumnWithValueOfVariable: "habCellArea"
                                      fromObject: self
                                        withType: "double"
                                       withLabel: "CellArea"];

  [cellFishInfoReporter addColumnWithValueOfVariable: "habCellDepth"
                                      fromObject: self
                                        withType: "double"
                                       withLabel: "CellDepth"];

  [cellFishInfoReporter addColumnWithValueOfVariable: "habCellVelocity"
                                      fromObject: self
                                        withType: "double"
                                       withLabel: "CellVelocity"];

  [cellFishInfoReporter addColumnWithValueOfVariable: "habCellDistToHide"
                                      fromObject: self
                                        withType: "double"
                                       withLabel: "CellDistToHide"];

  [cellFishInfoReporter addColumnWithValueOfVariable: "habCellFracShelter"
                                      fromObject: self
                                        withType: "double"
                                       withLabel: "CellFracShelter"];

  [cellFishInfoReporter breakOutUsingSelector: @selector(getSpecies)
                               withListOfKeys: speciesSymbolList];

  [cellFishInfoReporter breakOutUsingSelector: @selector(getAgeSymbol)
                               withListOfKeys: ageSymbolList];

  [cellFishInfoReporter createOutputWithLabel: "Count"
                                 withSelector: @selector(getFishCount)
                             withAveragerType: "Count"];

  [cellFishInfoReporter suppressColumnLabels: suppressBreakoutColumns];

  cellFishInfoReporter = [cellFishInfoReporter createEnd];

  //fprintf(stdout, "HabitatSpace >>>> buildCellFishInfoReporter >>>> END\n");
  //fflush(0);

  return self;
}


/////////////////////////////////////////
//
// outputCellFishInfoReport
//
////////////////////////////////////////
- outputCellFishInfoReport
{
   id <ListIndex> cellNdx = nil;
   id aCell = nil;

   //fprintf(stdout, "HabitatSpace >>>> %s >>>> outputCellFishInfoReport >>>> BEGIN\n", [reachSymbol getName]);
   //fflush(0);

   if(cellFishInfoReporter == nil)
   {
       fprintf(stderr, "ERROR: HabitatSpace >>>> outputCellFishInfoReport >>>> cellFishInfoReporter is nil\n");
       fflush(0);
       exit(1);
   }


   cellNdx = [listOfCells listBegin: scratchZone];


   while(([cellNdx getLoc] != End) && ((aCell = [cellNdx next]) != nil))
   {
      id <List> fishIContain = [aCell getFishIContain];
      id <ListIndex> fishNdx = [fishIContain listBegin: scratchZone];
      id aFish = nil;

      habCellDepth = [aCell getDepth];
      if(habCellDepth <= 0.0)
      {
          continue;
      }

      habCellTransectNumber = [aCell getTransect];
      habCellNumber = [aCell getCellNo];
      habCellArea = [aCell getArea];


      habCellVelocity = [aCell getCellVelocity];
      habCellDistToHide = [aCell getDistanceToHide];
      habCellFracShelter = [aCell getCellFracShelter];

      [cellFishList removeAll];

      if([cellFishList getCount] > 0)
      {
          fprintf(stderr, "ERROR: HabitatSpace >>>> outputCellFishInfoReport >>>> cellFishList is non-empty\n");
          fflush(0);
          exit(1);
      }

      while(([fishNdx getLoc] != End) && ((aFish = [fishNdx next]) != nil))
      {
          [cellFishList addLast: aFish];
      }
       
      [fishNdx drop];

      //
      // Now output the breakout reporter
      //
      [cellFishInfoReporter updateByReplacement];
      [cellFishInfoReporter output];
  }

  [cellNdx drop];

  //fprintf(stdout, "HabitatSpace >>>> %s >>>> outputCellFishInfoReport >>>> END\n", [reachSymbol getName]);
  //fflush(0);

  return self;
}





///////////////////////////////////
//
// printHabitatReport
//
///////////////////////////////////
- printHabitatReport 
{
  BOOL writeFileHeader;

  appendFiles = [model getAppendFiles];
  scenario = [model getScenario];
  replicate = [model getReplicate];

  writeFileHeader =    ((scenario == 1) && (replicate == 1));

  if(habitatRptFilePtr == NULL)
  {
      if(habitatReportFirstWrite == YES) 
      {
        if((appendFiles == 0) && (scenario == 1) && (replicate == 1))
        {
            if((habitatRptFilePtr = fopen(habitatReportFile,"w+")) == NULL)
            {
                fprintf(stderr, "ERROR: HabitatSpace >>>> printHabitatReport >>>> Cannot open file %s", habitatReportFile);
                fflush(0);
                exit(1);
            }
        }
        else  // Not appending files or not first scenario and replicate
        {
            if((habitatRptFilePtr = fopen(habitatReportFile,"a")) == NULL)
            {
               fprintf(stderr, "ERROR: HabitatSpace >>>> printHabitatReport >>>> Cannot open file %s",habitatReportFile);
               fflush(0);
               exit(1);
            }
        }

        if(writeFileHeader)
        {
            fprintf(habitatRptFilePtr,"%-12s%-12s%-12s%-20s%-20s%-20s%-20s%-12s%-12s\n", "Scenario",
                                                                  "Replicate",
                                                                  "Date",
                                                                  "DayLength", 
                                                        "YesterdaysRiverFlow", 
                                                            "TodaysRiverFlow", 
                                                         "TomorrowsRiverFlow", 
                                                                "Temperature", 
                                                                  "Turbidity");
            fflush(habitatRptFilePtr);
        }
      }  // if (habitatReportFirstWrite == YES )

     if(habitatReportFirstWrite == NO) 
     {
        if((habitatRptFilePtr = fopen(habitatReportFile,"a")) == NULL)
        {
           fprintf(stderr, "ERROR: HabitatSpace >>>> printHabitatReport >>>> Cannot open file %s",habitatReportFile);
           fflush(0);
           exit(1);
        }
     }

  }

  fprintf(habitatRptFilePtr,"%-12d%-12d%-12s%-20f%-20f%-20f%-20f%-12f%-12f\n",scenario,
                                                                              replicate,
                                             [timeManager getDateWithTimeT: modelTime_t], 
                                                                               dayLength, 
                                                                     yesterdaysRiverFlow, 
                                                                               riverFlow, 
                                                                      tomorrowsRiverFlow, 
                                                                             temperature, 
                                                                               turbidity);
  fflush(habitatRptFilePtr);
  habitatReportFirstWrite = NO;

  return self;
}





- printCellAreaDepthVelocityRpt
{

 FILE *depthVelPtr=NULL;

 if(depthVelRptFirstTime == YES) 
 {
   if( (depthVelPtr = fopen(cellAreaDepthVelReportFile, "w")) == NULL) 
   {
       fprintf(stderr, "ERROR: Cell >>>> printCellAreaDepthVelocityRpt >>>> Cannot open %s for writing\n", cellAreaDepthVelReportFile);
       fflush(0);
       exit(1);
   }
 }


 if(depthVelRptFirstTime == NO) 
 {
     if( (depthVelPtr = fopen(cellAreaDepthVelReportFile,"a")) == NULL) 
     {
          fprintf(stderr, "ERROR: Cell >>>> printCellAreaDepthVelocityRpt >>>> Cannot open %s for writing\n", cellAreaDepthVelReportFile);
          fflush(0);
          exit(1);
     }
 }

 [listOfCells forEach: M(depthVelReport:) : (id) depthVelPtr];


 fclose(depthVelPtr);
 depthVelRptFirstTime = NO;

 return self;
}



///////////////////////////////////////////////////////////////////
//
////              CELL REPORT METHODS
////// 
////////
///////////////////////////////////////////////////////////////////

- (BOOL) getFoodReportFirstTime {

   return foodReportFirstTime;

}

- setFoodReportFirstTime: (BOOL) aBool {

  foodReportFirstTime = aBool;

  return self;

}

- (BOOL) getDepthVelRptFirstTime {

  return depthVelRptFirstTime;

}


- setDepthVelRptFirstTime: (BOOL) aBool {

   depthVelRptFirstTime = aBool;

   return self;

}


///////////////////////////////////////////////////////////////
////
//////
////////   HYDRAULICS INITIALIZATION
/////////
////////////
///////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////
//
// readHydraulicInputFile
//
// This is used in a loop in HabitatManager
//
////////////////////////////////////////////////////////////
- readHydraulicInputFile: (char *) inputFile 
{

  if(hydraulicFileReader == nil)
  {
     hydraulicFileReader = [HydraulicFileReader createBegin: habitatZone];
  }

  [hydraulicFileReader readFile2: inputFile];

  [hydraulicFileReader checkStationOffsets];
  [hydraulicFileReader createEnd];

  stationList = [hydraulicFileReader getStationList];

  // 
  // if hydraulicFileReader is dropped here
  // then multiple calls to readHydraulicInputFile
  // could result in havoc
  // so drop it in setCellHydraulics after it is 
  // no longer needed
  //

  return self;

}

////////////////////////////////////////////////
//
// createCellInterpolationTables
//
///////////////////////////////////////////////
- createCellInterpolationTables
{
  [stationList forEach: M(createInterpolationTables)];
  return self;
}


//////////////////////////////////////////////////////
//
// setCellHydraulics
//
//////////////////////////////////////////////////////
- setCellHydraulics 
{
  int i, j;
  int maxCellOffset;
  int maxStationOffset;

  int numFlows;
  
  id aStation;
  id aCell;

  if(flowInterpolator == nil)
  {
    flowInterpolator = [InterpolationTable create: habitatZone];
  }

  flowArray = [hydraulicFileReader getFlowArray];

  if(stationList == nil) 
  {
     fprintf(stderr, "ERROR: HabitatSpace >>>> setCellHydraulics >>>> stationList is nil\n");
     fflush(0);
     exit(1);
  }

  maxCellOffset = [listOfCells getCount];
  maxStationOffset = [stationList getCount];

  if(maxCellOffset != maxStationOffset) 
  {
      fprintf(stderr, "ERROR: HabitatSpace >>>> setCellHydraulics >>>> listOfCells and stationList have different counts\n");
      fflush(0);
      exit(1);
  }

  for(i=0 ; i < maxStationOffset; i++) 
  {
    aStation = [stationList atOffset: i];   
    for(j=0; j < maxCellOffset; j++) 
    {
       aCell = [listOfCells atOffset: j];

       if( ([aStation getCellNo] == [aCell getCellNo]) && ([aStation getTransect] == [aCell getTransect]) ) 
       {
          [aCell setHydStation: aStation];
          [aCell setBottomElev: [aStation getBottomElev]];
          [aCell setVelocityInterpolator: [aStation getVelocityInterpolator]];
          [aCell setWslInterpolator: [aStation getWslInterpolator]];
       }
     }
  } 

  if(flowArray == nil) 
  {
      fprintf(stderr, "ERROR: HabitatSpace >>>> setCellHydraulics >>>> flowArray is nil\n");
      fflush(0);
      exit(1);
  }
  

  //
  // Now, create the flow interpolator for the habitatSpace;
  // used to calculate the index and interpolation fraction for 
  // all of the cell interpolators
  // 
  numFlows = [flowArray getCount];
  for(i=0; i < numFlows; i++) 
  {
      double aFlow = *((double *) [flowArray atOffset: i]);
      double dummyValue = 5.1;

      [flowInterpolator addX: aFlow Y: dummyValue];
  }

  [listOfCells forEach: M(checkThatHydStationIsNotNil)];

  // 
  // hydraulicFileReader is no longer needed...
  //
  [hydraulicFileReader drop];  //note to self: what happens to stationList flowArray??
  hydraulicFileReader = nil;

  return self;

}














//////////////////////////////////////////////////////////////
//
////
//////  BARRIERS
///////
/////////
//////////////////////////////////////////////////////////////

////////////////////////////////////////////////
//
// buildBarriersFromList
//
////////////////////////////////////////////////
- buildBarriersFromList: (id <List>) aBarrierXList
{

  int i;
  int barrierListCount;

  fprintf(stdout, "HabitatSpace >>>> buildBarriersFromList >>>> BEGIN\n");
  fflush(0);

  if(aBarrierXList == nil)
  {
     fprintf(stderr, "ERROR: HabitatSpace >>>> buildBarriersFromList >>>> aBarrierXList is nil\n");
     fflush(0);
  }

  barrierListCount = [aBarrierXList getCount];

  for(i = 0; i < barrierListCount; i++)
  {
     Barrier *barrier=nil;
     double* aBarrierX = (double *) [aBarrierXList atOffset: i];

     barrier = [Barrier create: habitatZone];
     [barrier setBarrierX: *aBarrierX];              //gets multiplied by 100 by the barrier
     [barrier setMinY: minCellY andMaxY: maxCellY];
     [barrierList addLast: barrier];
  }

  barrierCount = [barrierList getCount];

  fprintf(stdout, "HabitatSpace >>>> buildBarriersFromList >>>> END\n");
  fflush(0);

  return self;
}


//////////////////////////////////////////////////////////////////
//
// setBarrierRasterResolutionX
//
/////////////////////////////////////////////////////////////////
- setBarrierRasterResolutionX: (int) aResolutionX 
               andResolutionY: (int) aResolutionY {

  id <ListIndex> barrierNdx;
  id aBarrier=nil;

  if(barrierCount != 0 ) {

      barrierNdx = [barrierList listBegin: scratchZone];

      while( ([barrierNdx getLoc] != End) && 
             ((aBarrier = [barrierNdx next]) != nil) ) {

        [aBarrier setBarrierRasterResolutionX: aResolutionX 
                               andResolutionY: aResolutionY];
    
      }

  [barrierNdx drop];

  }

 
  return self;

}


//////////////////////////////////////////////////////////////////
//
// getBarriers
//
/////////////////////////////////////////////////////////////////
- (id <List>) getBarriers {

  return barrierList;

}



//////////////////////////////////////////////////////////////////
//
// isThereABarrierTo
//
// returns 0 if there is a barrier upstream of myCell
//         1 if there is a barrier downstream 
//        -1 if there is no barrier
//
/////////////////////////////////////////////////////////////////
- (int) isThereABarrierTo: aDestCell from: myCell 
{
  id aBarrier=nil;
  int barrierNdx;

  double myMidX;
  double destMidX; 
  double barrierX;

  id destCellReach = [aDestCell getReach];
  id myCellReach = [myCell getReach];

  int barrierVal = -1;         // This needs to be initialized to -1
                              
  if(myCellReach != self) 
  {
      fprintf(stderr, "ERROR: HabitatSpace >>>> isThereABarrierTo >>>> myCellReach is not self\n");
      fflush(0);
      exit(1);
  }


  if(myCellReach == destCellReach)
  {
      if(barrierCount != 0 ) 
      {

          myMidX   = [[myCell getMidPoint] getElement: XCOORDINATE];
          destMidX = [[aDestCell getMidPoint] getElement: XCOORDINATE];

            for(barrierNdx = 0; barrierNdx < barrierCount; barrierNdx++) 
            {
                aBarrier = [barrierList atOffset: barrierNdx];

                if(aBarrier == nil) continue;

                barrierX = [aBarrier getBarrierX];

                if( (myMidX < barrierX) && (barrierX <= destMidX) ) 
                {
                      barrierVal = 0;   //barrier is upstream to myCell
                      break;
                }
                else if( (myMidX > barrierX) && (barrierX >= destMidX) ) 
                {
                      barrierVal = 1;   //barrier is downstream to myCell
                      break;
                }

            } // for barrierNdx
       } //if barrierCount

       return barrierVal;
   }

   //
   // aDestCell is in a different reach than myCell -
   // so look at both reaches
   // Return 0 if there are any upstream barriers in either reach,
   // else return 1 if there are any downstream barriers in either reach
   //
   else if([habDownstreamLinksToUS contains: destCellReach])
   {
        if([myCellReach hasBarrierBetweenDownstreamEndAnd: myCell] == YES)
        {
            return 1;
        }

        if([destCellReach hasBarrierBetweenUpstreamEndAnd: aDestCell] == YES)
        {
            return 1;
        }

        return -1;
   }

   else if([habDownstreamLinksToDS contains: destCellReach])
   {
        if([myCellReach hasBarrierBetweenDownstreamEndAnd: myCell] == YES)
        {
            barrierVal = 1;
        }

        if([destCellReach hasBarrierBetweenDownstreamEndAnd: aDestCell] == YES)
        {
            barrierVal = 0;
        }

        return barrierVal;
   }

   else if([habUpstreamLinksToDS contains: destCellReach])
   {
        if([myCellReach hasBarrierBetweenUpstreamEndAnd: myCell] == YES)
        {
           return 0;
        }

        if([destCellReach hasBarrierBetweenDownstreamEndAnd: aDestCell] == YES)
        {
           return 0;
        }

        return -1;

   }

   else if([habUpstreamLinksToUS contains: destCellReach])
   {

        if([myCellReach hasBarrierBetweenUpstreamEndAnd: myCell] == YES)
        {
            return 0;
        }

        if([destCellReach hasBarrierBetweenUpstreamEndAnd: aDestCell] == YES)
        {
            return 1;
        }
        
        return -1;

   }

   else
   {
       fprintf(stderr, "ERROR: HabitatSpace >>>> isThereABarrierTo >>>> aDestCell was not foundd in any of the linked reaches\n");
       fflush(0);
       exit(1);
   }

  //
  // We should never reach this point in the execution of the code
  //
  fprintf(stderr, "ERROR: HabitatSpace >>>> isThereABarrierTo >>>> barrierVal\n");
  fflush(0);
  exit(1);

  return barrierVal;
}


- (BOOL) hasBarrierBetweenDownstreamEndAnd: aCell
{

    double cellMidX;
    int barrierNdx;

    if([listOfCells contains: aCell] == NO)
    {
        [InternalError raiseEvent: "ERROR: HabitatSpace >>>> hasBarrierBetweenDownstreamEndAnd >>>> aCell has to be in the habitat space executing this method\n"];
    }

    if(barrierCount == 0) return NO;

    cellMidX = [[aCell getMidPoint] getElement: XCOORDINATE];

    for(barrierNdx = 0; barrierNdx < barrierCount; barrierNdx++)
    { 
       id aBarrier = [barrierList atOffset: barrierNdx];
       double barrierX;

       if(aBarrier == nil) continue;

       barrierX = [aBarrier getBarrierX];

       if(cellMidX > barrierX)
       {
          return YES; // barrier is down stream of aCell
       } 
    } 
       
    return NO;

}



- (BOOL) hasBarrierBetweenUpstreamEndAnd: aCell
{

    double cellMidX;
    int barrierNdx;

    if([listOfCells contains: aCell] == NO)
    {
        [InternalError raiseEvent: "ERROR: HabitatSpace >>>> hasBarrierBetweenUpstreamEndAnd >>>> aCell has to be in the habitat space executing this method\n"];
    }

    if(barrierCount == 0) return NO;

    cellMidX = [[aCell getMidPoint] getElement: XCOORDINATE];

    for(barrierNdx = 0; barrierNdx < barrierCount; barrierNdx++)
    { 
       id aBarrier = [barrierList atOffset: barrierNdx];
       double barrierX;

       if(aBarrier == nil) continue;

       barrierX = [aBarrier getBarrierX];

       if(cellMidX < barrierX)
       {
          return YES; // barrier is upstream of aCell
       }
  
    } 
       

    return NO;

}





////////////////////////////////////////////////////////////////////
//
////                       
//////              PRED DENSITY
///////
/////////
//////////
////////////////////////////////////////////////////////////////////

///////////////////////////////////////
//
// incrementNumPiscivorousFish
//
//////////////////////////////////////
- incrementNumPiscivorousFish
{
   numPiscivorousFish++;


   return self;
}


////////////////////////////////////////
//
// updateHabSurvProbForAqPred
//
////////////////////////////////////////
- updateHabSurvProbForAqPred
{
   [listOfCells forEach: M(updateHabSurvProbForAqPred)];

   return self;
}

/////////////////////////////////////////
//
// calcPiscivorousFishDensity
//
////////////////////////////////////////
- (double) calcPiscivorousFishDensity
{
   piscivorousFishDensity = 0.0;

   if(habWettedArea > 0.0)
   {
       piscivorousFishDensity = numPiscivorousFish/habWettedArea;
   }

   return piscivorousFishDensity;
}

///////////////////////////////////
//
// getPiscivorousFishDensity
//
/////////////////////////////////
- (double) getPiscivorousFishDensity
{
   return piscivorousFishDensity;
}




/////////////////////////////////////////////////////////////////
//
////            HISTOGRAM OUTPUT
//////
////////     Note: the set and open messages are sent from the
//////////         *Observer* Swarm NOT the model swarm
////////////
/////////////////////////////////////////////////////////////////

- setAreaDepthBinWidth: (int) aWidth {

  depthBinWidth = aWidth;

  return self;

}


//////////////////////////////////////////////////////////
//
// setAreaVelocityBinWidth
//
//////////////////////////////////////////////////////////
- setAreaVelocityBinWidth: (int) aWidth {

  velocityBinWidth = aWidth;

  return self;

}


///////////////////////////////////////////////////////////
//
// setDepthHistoMaxDepth
//
/////////////////////////////////////////////////////////
- setDepthHistoMaxDepth: (double) aDepth {

  depthHistoMaxDepth = aDepth;

  return self;

}

///////////////////////////////////////////////////////
//
// setVelocityHistoMaxVelocity
//
//////////////////////////////////////////////////////
- setVelocityHistoMaxVelocity: (double) aVelocity {
 
  velocityHistoMaxVelocity = aVelocity;

  return self;

}


///////////////////////////////////////////////////////
//
// setAreaDepthHistoFmtStr
//
///////////////////////////////////////////////////////
- setAreaDepthHistoFmtStr: (char *) aFmtStr {

  areaDepthHistoFmtStr = aFmtStr;

  return self;
 
}

///////////////////////////////////////////////////////
//
// setAreaVelocityHistoFmtStr
//
///////////////////////////////////////////////////////
- setAreaVelocityHistoFmtStr: (char *) aFmtStr {

  areaVelocityHistoFmtStr = aFmtStr;

  return self;
 
}

//////////////////////////////////////////////////////////
//
// openAreaDepthFile
//
//////////////////////////////////////////////////////////
- openAreaDepthFile: (char *) aFileName {
  int maxBinNumber= (unsigned) floor(depthHistoMaxDepth/depthBinWidth);
  int i;

  if([model getAppendFiles] == NO) {
      if( (areaDepthFileStream = fopen(aFileName, "w")) == NULL) {

          [InternalError raiseEvent: "ERROR: Cannot open %s in habitatSpace \n", aFileName];

      }

      fprintf(areaDepthFileStream, "%-14s%-10s%-11s","Date", "Scenario", "Replicate");
      fflush(0);

      for(i=0;i < maxBinNumber; i++) {

        //fprintf(areaDepthFileStream, "%-10d",(i+1)*depthBinWidth);
        fprintf(areaDepthFileStream, areaDepthHistoFmtStr,(i+1)*depthBinWidth);
        fflush(0);

      } //for

        fprintf(areaDepthFileStream, ">");
        fprintf(areaDepthFileStream, areaDepthHistoFmtStr, maxBinNumber*depthBinWidth);
        fprintf(areaDepthFileStream, "\n");
        fflush(0);

  }
  else {

     if( ([model getScenario] == 1) && ([model getReplicate] == 1) ) {
        if( (areaDepthFileStream = fopen(aFileName, "w")) == NULL) {

            [InternalError raiseEvent: "ERROR: Cannot open %s in habitatSpace \n", aFileName];

        }

        fprintf(areaDepthFileStream, "%-14s%-10s%-11s","Date", "Scenario", "Replicate");
        fflush(0);

        for(i=0;i < maxBinNumber; i++) {

           fprintf(areaDepthFileStream, areaDepthHistoFmtStr,(i+1)*depthBinWidth);
           fflush(0);

        } //for

        fprintf(areaDepthFileStream, ">");
        fprintf(areaDepthFileStream, areaDepthHistoFmtStr, maxBinNumber*depthBinWidth);
        fprintf(areaDepthFileStream, "\n");
        fflush(0);
    }
    else {

        if((areaDepthFileStream = fopen(aFileName, "a")) == NULL) {

           [InternalError raiseEvent: "ERROR: Cannot open %s in habitatSpace \n", aFileName];
        }

    }

   }

  return self;

}


//////////////////////////////////////////////////////////
//
// openAreaVelocityFile
//
//////////////////////////////////////////////////////////
- openAreaVelocityFile: (char *) aFileName {
  int maxBinNumber= (unsigned) floor(velocityHistoMaxVelocity/depthBinWidth);
  int i;


  if([model getAppendFiles] == NO) {
      if( (areaVelocityFileStream = fopen(aFileName, "w")) == NULL) {

          [InternalError raiseEvent: "ERROR: Cannot open %s in habitatSpace \n", aFileName];

      }

     fprintf(areaVelocityFileStream, "%-14s%-10s%-11s","Date", "Scenario", "Replicate");
     fflush(0);

      for(i=0;i < maxBinNumber; i++) {

        fprintf(areaVelocityFileStream, areaVelocityHistoFmtStr ,(i+1)*velocityBinWidth);
        fflush(0);

      } //for

        fprintf(areaVelocityFileStream, ">");
        fprintf(areaVelocityFileStream, areaVelocityHistoFmtStr ,maxBinNumber*velocityBinWidth);
        fprintf(areaVelocityFileStream, "\n");
        fflush(0);

  }
  else {

     if( ([model getScenario] == 1) && ([model getReplicate] == 1) ) {
        if( (areaVelocityFileStream = fopen(aFileName, "w")) == NULL) {

            [InternalError raiseEvent: "ERROR: Cannot open %s in habitatSpace \n", aFileName];

        }

        fprintf(areaVelocityFileStream, "%-14s%-10s%-11s","Date", "Scenario", "Replicate");
        fflush(0);

        for(i=0;i < maxBinNumber; i++) {

           fprintf(areaVelocityFileStream, areaVelocityHistoFmtStr ,(i+1)*velocityBinWidth);
           fflush(0);

        } //for

        fprintf(areaVelocityFileStream, ">");
        fprintf(areaVelocityFileStream, areaVelocityHistoFmtStr ,maxBinNumber*velocityBinWidth);
        fprintf(areaVelocityFileStream, "\n");
        fflush(0);
    }
    else {

        if((areaVelocityFileStream = fopen(aFileName, "a")) == NULL) {

           [InternalError raiseEvent: "ERROR: Cannot open %s in habitatSpace \n", aFileName];
        }

    }

   }

  return self;

}
  

//////////////////////////////////////////////////////////
//
// printAreaDepthHisto
//
//////////////////////////////////////////////////////////
- printAreaDepthHisto {
  id <ListIndex> cellNdx;
  id cell=nil;

  double cellDepth;
  double cellArea;

  int depthBinNumber;
  int maxBinNumber= (unsigned) floor(depthHistoMaxDepth/depthBinWidth);
  int depthAreaBin[maxBinNumber + 1];
  int i;

  for(i=0;i <= maxBinNumber;i++) {

     depthAreaBin[i] = 0.0;

  }

  cellNdx = [listOfCells listBegin: habitatZone];

  while( ([cellNdx getLoc] != End) && ((cell = [cellNdx next]) != nil) ) {
     cellDepth = [cell getDepth];
     cellArea = [cell getArea];

     if( (cellDepth > 0.0) && (cellDepth < depthHistoMaxDepth) ) {
         depthBinNumber = (unsigned) floor(cellDepth/depthBinWidth);
         depthAreaBin[depthBinNumber] += (int) cellArea;
     }
     else if(cellDepth > depthHistoMaxDepth) {

         depthAreaBin[maxBinNumber] += (int) cellArea;

     }


  }

  [cellNdx drop];
  
  scenario = [model getScenario];
  replicate = [model getReplicate];

  fprintf(areaDepthFileStream, "%-14s",[timeManager getDateWithTimeT: modelTime_t]);
  fprintf(areaDepthFileStream, "%-10d%-11d",scenario, replicate);
  fflush(0);
  
  for(i=0;i <= maxBinNumber; i++) {

    fprintf(areaDepthFileStream, areaDepthHistoFmtStr ,depthAreaBin[i]);
    fflush(0);

  }

  fprintf(areaDepthFileStream,"\n");
  fflush(0);

  return self;

}


//////////////////////////////////////////////////////////
//
//
//
//////////////////////////////////////////////////////////

- printAreaVelocityHisto {
  id <ListIndex> cellNdx;
  id cell=nil;

  double cellVelocity;
  double cellArea;

  int velocityBinNumber;
  int maxBinNumber= (unsigned) floor(velocityHistoMaxVelocity/velocityBinWidth);
  int velocityAreaBin[maxBinNumber + 1];
  int i;

  for(i=0;i <= maxBinNumber;i++) {

     velocityAreaBin[i] = 0.0;

  }

  cellNdx = [listOfCells listBegin: habitatZone];

  while( ([cellNdx getLoc] != End) && ((cell = [cellNdx next]) != nil) ) {
     cellVelocity = [cell getCellVelocity];
     cellArea = [cell getArea];

     if( (cellVelocity > 0.0) && (cellVelocity < velocityHistoMaxVelocity) ) {
         velocityBinNumber = (int) floor(cellVelocity/velocityBinWidth);
         velocityAreaBin[velocityBinNumber] += (int) cellArea;
     }
     else if(cellVelocity > velocityHistoMaxVelocity) {

         velocityAreaBin[maxBinNumber] += (int) cellArea;

     }


  }

  [cellNdx drop];
  
  scenario = [model getScenario];
  replicate = [model getReplicate];

  fprintf(areaVelocityFileStream, "%-14s",[timeManager getDateWithTimeT: modelTime_t]);
  fprintf(areaVelocityFileStream, "%-10d%-11d",scenario, replicate);
  fflush(0);
  
  for(i=0;i <= maxBinNumber; i++) {

    fprintf(areaVelocityFileStream, areaVelocityHistoFmtStr, velocityAreaBin[i]);
    fflush(0);

  }

  fprintf(areaVelocityFileStream,"\n");
  fflush(0);

  return self;

}
















/////////////////////////////////////////////////
//
////       CLEANUP
//////
////////
/////////
////////////////////////////////////////////////
- (void) drop 
{
     fclose(areaDepthFileStream);
     fclose(areaVelocityFileStream);

    
    if(cellFishInfoReporter != nil)
    {
        [cellFishInfoReporter drop];
        cellFishInfoReporter = nil;
    }
    if(habitatRptFilePtr != NULL) 
    {
        fclose(habitatRptFilePtr);
    }

    [habitatZone drop];
}


@end
