/*
inSTREAM Version 4.2, October 2006.
Individual-based stream trout modeling software. Developed and maintained by Steve Railsback (Lang, Railsback & Associates, Arcata, California) and
Steve Jackson (Jackson Scientific Computing, McKinleyville, California).
Development sponsored by EPRI, US EPA, USDA Forest Service, and others.
Copyright (C) 2004 Lang, Railsback & Associates.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program (see file LICENSE); if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*/




#import "TroutObserverSwarm.h"
#import "TroutModelSwarm.h"
#import <collections.h>
#import <objectbase.h>
#import <gui.h>




@implementation TroutObserverSwarm

//////////////////////////////////////////////
//
// create
//
//////////////////////////////////////////
+ create: aZone
{
  TroutObserverSwarm* obj=nil;

  obj = [super create: aZone];

  obj->finished=NO;
  obj->probeMap=nil;
  obj->modelNumber = 1;
  obj->rasterOutputFrequency = 1;
  obj->displayFrequency = 1;
  [obj buildProbesIn: aZone];

  return obj;
}


/////////////////////////////////////////////////////////////////
//
// getModel
//
//////////////////////////////////////////////////////////////
- getModel 
{
  return troutModelSwarm;
}



////////////////////////////////////////////////////////////////
//
// buildProbes
//
///////////////////////////////////////////////////////////////
- buildProbesIn: aZone 
{

  /* Put the probe design for each class to be observed here.*/

  /*
   
  //
  // Probemap for the observer swarm is disabled
  //

  probeMap = [CustomProbeMap create: aZone forClass: [TroutObserverSwarm class]
			     withIdentifiers: "displayFrequency",
			     "rasterZoomFactor",
			     "rasterOutputFrequency",
			     "takeRasterPictures",
			     ":",
			     "iAmAlive", NULL];
  [probeLibrary setProbeMap: probeMap For: [TroutObserverSwarm class]];
  */



  /*

  //
  // Probemap for the ModelSwarm is disabled
  //

  probeMap = [CustomProbeMap create: aZone forClass: [TroutModelSwarm class]
			     withIdentifiers: "fpm",
			     "speciesSymbolList",
			     "totalTroutPopList",
			     "habitatSpace",
                             "numTrout",
			     ":",
                             "getReddList",
                             "getTotalTroutPopList",
                             "getDeadTroutList",
                             "getCachedTroutList",
                             ":",
                             NULL];
  [probeLibrary setProbeMap: probeMap For: [TroutModelSwarm class]];
  */

  //
  // HabitatSpace
  //
  probeMap = [CustomProbeMap create: aZone forClass: [HabitatSpace class]
			     withIdentifiers: "reachName",
                             "Date",
                             "temperature",
                             "habWettedArea",
                             "riverFlow",
                             "turbidity",
                             "dayLength",
                             "habDownstreamJunctionNumber",
                             "habUpstreamJunctionNumber",
                             ":",
                             NULL];
  [probeLibrary setProbeMap: probeMap For: [HabitatSpace class]];

  //
  // Cells
  //
  probeMap = [CustomProbeMap create: aZone forClass: [Cell class]
			     withIdentifiers: "transectNumber",
                             "cellNumber",
                             "velocity",
			     "depth",
                             "area",
                             "cellShelterArea",
                             "shelterAreaAvailable",
                             "driftHourlyCellTotal",
                             "hourlyAvailDriftFood",
                             "searchHourlyCellTotal",
                             "hourlyAvailSearchFood",
                             "cellFracSpawn",
                             "cellDistToHide",
			     ":",
                             "getNumberOfFish",
                             "getNumberOfRedds",
                             "getHabSearchProd",
                             "getHabDriftConc",
                             "getHabDriftRegenDist",
                             "getHabPreyEnergyDensity",
                             NULL];
  [probeLibrary setProbeMap: probeMap For: [Cell class]];

  //
  // The fish probes are defined in buildFishProbes
  //


  //
  // Redds
  //
  probeMap = [CustomProbeMap create: aZone forClass: [TroutRedd class]
			     withIdentifiers: "Species",
			     "numberOfEggs",
                             "emergeDays",
                             "fracDeveloped",
                             "spawnerLength",
                             "transect",
                             "cellNo",
                             ":",
                             NULL];
  [probeLibrary setProbeMap: probeMap For: [TroutRedd class]];

    return self;
}




////////////////////////////////////////////////////////////////////
//
// buildFishProbes
//
///////////////////////////////////////////////////////////////////
- buildFishProbes 
{
  id <ListIndex> listNdx;
  id speciesClass;

  probeMap = [CustomProbeMap create: [self getZone] forClass: [Trout class]
			   withIdentifiers: "age",
					    "fishLength",
					    "fishWeight",
					    "fishCondition",
					    //"prevLength",
					    //"prevWeight",
					    //"prevCondition",
					    //"netEnergyForBestCell",
					    //"dailyDriftFoodIntake",
					    //"dailyDriftNetEnergy",
					    //"dailySearchFoodIntake",
					    //"dailySearchNetEnergy",
					    //"feedTime",
					    //"potentialHourlyDriftIntake",
					    //"potentialHourlySearchIntake",
					    //"cMax",
					    //"standardResp",
					    //"activeResp",
					    "feedStrategy",
					    "inShelter",
					    "deadOrAlive",
					    "deathCausedBy",
					    "captureSuccess",
					    "captureArea",
					    "nonStarvSurvival",
					    ":",
					    "tagFish",
					    "tagCellsICouldMoveTo",
					    "makeMeImmortal",
                                            "killFish",
					     NULL];

   listNdx = [[troutModelSwarm getSpeciesClassList] listBegin: scratchZone];
   while (([listNdx getLoc] != End) && ((speciesClass = [listNdx next]) != nil)) 
   {
       [probeLibrary setProbeMap: probeMap For: speciesClass];
   }
   [listNdx drop];
[probeDisplayManager setDropImmediatelyFlag: NO];
   return self;
}






- _worldRasterDeath_ : caller
{
  //[worldRaster drop];
  worldRaster = nil;
  return self;
}

- _velocityHistoDeath_ : caller
{
  velocityHisto = nil;
  return self;
}

- _depthHistoDeath_ : caller
{
  depthHisto = nil;
  return self;
}

- _populationHistoDeath_ : caller
{
  //[populationHisto drop];
  populationHisto = nil;
  return self;
}

- _mortalityGraphDeath : caller
{
  //[mortalityGraph drop];
  mortalityGraph = nil;
  return self;
}



//////////////////////////////////////////////////
//
// objectSetup
//
/////////////////////////////////////////////////
- objectSetup 
{

  obsZone = [Zone create: [self getZone]];

  troutModelSwarm = [TroutModelSwarm create: self];
  [troutModelSwarm setObserverSwarm: self];

  [ObjectLoader load: troutModelSwarm fromFileNamed: "Model.Setup"];

  troutModelSwarm->rasterResolution  = rasterResolution;
  troutModelSwarm->rasterResolutionX = rasterResolutionX;
  troutModelSwarm->rasterResolutionY = rasterResolutionY;
  troutModelSwarm->rasterColorVariable = rasterColorVariable;

  //
  // Instantiate the objects first;
  // this allows the experiment swarm to operate on 
  // model objects BEFORE their final creation 
  //
  [troutModelSwarm instantiateObjects];



  return self;
}



////////////////////////////////////////////////////////
//
// buildObjects
//
///////////////////////////////////////////////////////
- buildObjects
{
  int ndx;
  unsigned rasterX, rasterY, rasterSize;

  if((rasterResolution <= 0) || (rasterResolutionX <= 0) || (rasterResolutionY <= 0))
  {
     fprintf(stderr, "TroutObserverSwarm >>>> buildObjects >>>> one of the rasterResolution parameters is <= zero\n");
     fflush(0);
     exit(1);
  }

  colormap = [Colormap create: globalZone];
  for (ndx = 0; ndx <= CELL_COLOR_MAX; ndx++)  
  {
 
    [colormap setColor: ndx 
                 ToRed: (CELL_COLOR_START + 
                        ((double)ndx * 
                        ((1.0-CELL_COLOR_START)/((double)CELL_COLOR_MAX))))
                 Green: (CELL_COLOR_START + 
                        ((double)ndx * 
                        ((1.0-CELL_COLOR_START)/((double)CELL_COLOR_MAX))))
                  Blue: (CELL_COLOR_START + 
                        ((double)ndx * 
                        ((1.0-CELL_COLOR_START)/((double)CELL_COLOR_MAX))))];


   }

  [colormap setColor: BARRIER_COLOR ToName: "white"];
  [colormap setColor: TAG_CELL_COLOR ToName: "aquamarine"];
  
  //
  //build model Objects and set the fish color in the ModelSwarm
  //
  [troutModelSwarm buildObjects: colormap];


  [self buildFishProbes];


  //
  // Build the rasters, display objects, etc from the 
  // HabitatManager 
  // 
  {
       int numberOfSpaces = -1;
       int spaceCount;
       id habitatManager = nil;
       id <List> habSpaceList;

       habitatManager = [troutModelSwarm getHabitatManager];    
       numberOfSpaces = [habitatManager getNumberOfHabitatSpaces];
       habSpaceList = [habitatManager getHabitatSpaceList];
       
       habitatRasterList = [List create: obsZone];
       habCellDisplayList = [List create: obsZone];
       habBarrierDisplayList = [List create: obsZone];

       fprintf(stdout, "TroutObserverSwarm >>>> buildObjects >>>> building space display objects >>>> BEGIN\n");
       fflush(0);


       for(spaceCount = 0; spaceCount < numberOfSpaces; spaceCount++)
       {
            id habitatSpace = [habSpaceList atOffset: spaceCount];
             
            id <ZoomRaster> habWorldRaster;          // 2d display widget
            id <Object2dDisplay> habCellDisplay;	  // display the trout
            id <Object2dDisplay> habBarrierDisplay;  // display the Barriers


            xprint(habitatSpace);


            habWorldRaster = [ZoomRaster createBegin: obsZone];
            SET_WINDOW_GEOMETRY_RECORD_NAME (habWorldRaster);
            habWorldRaster = [habWorldRaster createEnd];
            [habWorldRaster enableDestroyNotification: self
	                 notificationMethod: @selector (_worldRasterDeath_:)];

            [habWorldRaster setColormap: colormap];

            rasterX = [habitatSpace getPixelsX];
            rasterY = [habitatSpace getPixelsY];
            rasterSize = (rasterX >= rasterY ? rasterX : rasterY )/rasterResolution;
            [habWorldRaster setZoomFactor: rasterZoomFactor];

            [habWorldRaster setWidth: rasterX/rasterResolutionX Height: rasterY/rasterResolutionY];

            [habWorldRaster setWindowTitle: [habitatSpace getReachName]];

            [habWorldRaster pack];				  // draw the window.

            habCellDisplay = [Object2dDisplay createBegin: obsZone];
            [habCellDisplay setDisplayWidget: habWorldRaster];
            [habCellDisplay setDiscrete2dToDisplay: habitatSpace];
            [habCellDisplay setObjectCollection: 
		            [habitatSpace getCells]];
            [habCellDisplay setDisplayMessage: M(drawSelfOn:)];   // draw method
            habCellDisplay = [habCellDisplay createEnd];

            habBarrierDisplay = [Object2dDisplay createBegin: obsZone];
            [habBarrierDisplay setDisplayWidget: habWorldRaster];
            [habBarrierDisplay setDiscrete2dToDisplay: habitatSpace];
            [habBarrierDisplay setObjectCollection: 
		[habitatSpace getBarriers]];
            [habBarrierDisplay setDisplayMessage: M(drawSelfOn:)];   // draw method
            habBarrierDisplay = [habBarrierDisplay createEnd];


            [habWorldRaster setButton: ButtonLeft
	                 Client: habitatSpace 
	                 Message: M(probeCellAtX:Y:)];

            [habWorldRaster setButton: ButtonRight
	                 Client: habitatSpace 
	                 Message: M(probeFishAtX:Y:)];



            [habitatRasterList addLast: habWorldRaster];
            [habCellDisplayList addLast: habCellDisplay];
            [habBarrierDisplayList addLast: habBarrierDisplay];


         } //for

         fprintf(stdout, "TroutObserverSwarm >>>> buildObjects >>>> building space display objects >>>> END\n");
         fflush(0);

   } // Build Display Objects


  velocityHisto = [EZBin createBegin: obsZone];
  SET_WINDOW_GEOMETRY_RECORD_NAME (velocityHisto);
  [velocityHisto setTitle: "# of Fish for each Velocity"];
  [velocityHisto setAxisLabelsX: "Velocity (cm)" Y: "# of Fish"];
  [velocityHisto setBinCount: 10];
  [velocityHisto setLowerBound: 0];
  [velocityHisto setUpperBound: 100];
  [velocityHisto setCollection: [troutModelSwarm getLiveFishList]];
  [velocityHisto setProbedSelector: M(getCellVelocity)];
  [velocityHisto setFileOutput: YES];
  [velocityHisto setFileName: "FishVelocityHisto.out"];
  velocityHisto = [velocityHisto createEnd];
  [velocityHisto enableDestroyNotification: self
                notificationMethod: @selector (_velocityHistoDeath_:)];


  depthHisto = [EZBin createBegin: obsZone];
  SET_WINDOW_GEOMETRY_RECORD_NAME (depthHisto);
  [depthHisto setTitle: "# of Fish for each Depth"];
  [depthHisto setAxisLabelsX: "Depth (cm)" Y: "# of Fish"];
  [depthHisto setBinCount: 10];
  [depthHisto setLowerBound: 0];
  [depthHisto setUpperBound: 100];
  [depthHisto setCollection: [troutModelSwarm getLiveFishList]];
  [depthHisto setProbedSelector: M(getCellDepth)];
  [depthHisto setFileOutput: YES];
  [depthHisto setFileName: "FishDepthHisto.out"];
  depthHisto = [depthHisto createEnd];
  [depthHisto enableDestroyNotification: self
                notificationMethod: @selector (_depthHistoDeath_:)];


  populationHisto = [EZBin createBegin: obsZone];
  SET_WINDOW_GEOMETRY_RECORD_NAME (populationHisto);
  [populationHisto setTitle: "Population in each Age"];
  [populationHisto setAxisLabelsX: "Age" Y: "# of Fish"];
  [populationHisto setBinCount: 5];
  [populationHisto setLowerBound: 0];
  [populationHisto setUpperBound: 5];
  [populationHisto setCollection: [troutModelSwarm getLiveFishList]];
  [populationHisto setProbedSelector: M(getAge)];
  populationHisto = [populationHisto createEnd];
  [populationHisto enableDestroyNotification: self
                notificationMethod: @selector (_populationHistoDeath_:)];


  mortalityGraph = [EZGraph createBegin: self];
  SET_WINDOW_GEOMETRY_RECORD_NAME (mortalityGraph); 
  [mortalityGraph setTitle: "Mortality"];
  [mortalityGraph setAxisLabelsX: "Time" Y: "NumberDead"];
  mortalityGraph = [mortalityGraph createEnd];

  //
  // Now create the graph sequences
  //
  {
      id <List> listOfMortalityCounts = [troutModelSwarm getListOfMortalityCounts];
      id <ListIndex> lstNdx = nil;
      id mortalityCount = nil;

      if(listOfMortalityCounts == nil) 
      {
          fprintf(stderr, "ERROR: TroutObserverSwarm >>>> buildObjects >>>> listOfMortalityCounts is nil\n");
          fflush(0);
          exit(1);
      }
  
      lstNdx = [listOfMortalityCounts listBegin: scratchZone];

      [lstNdx setLoc: Start];

      while(([lstNdx getLoc] != End) && ((mortalityCount = [lstNdx next]) != nil)) 
      {
            [mortalityGraph createSequence: [[mortalityCount getMortality] getName]
                              withFeedFrom: mortalityCount
                               andSelector: M(getNumDead)];
      }

      [lstNdx drop];
  }

  [mortalityGraph enableDestroyNotification: self
               notificationMethod: @selector (_mortalityGraphDeath:)];



  //
  // One for each habitat space
  //
  if(troutModelSwarm)
  {
      id <List> aHabitatSpaceList = [[troutModelSwarm getHabitatManager] getHabitatSpaceList]; 
      int habitatSpaceCount = [aHabitatSpaceList getCount];
      int i;

      for(i = 0; i < habitatSpaceCount; i++)
      {
          CREATE_ARCHIVED_PROBE_DISPLAY([aHabitatSpaceList atOffset: i]);
      }
  }

  return self;
}  

- _update_ 
{

  if (worldRaster) 
  {
      [worldRaster erase];
      [cellDisplay display];
      [barrierDisplay display];
      [worldRaster drawSelf];
  }
  if (depthHisto) 
  {
    [depthHisto reset];
    [depthHisto update];
    [depthHisto output];
  }  
  if(velocityHisto) 
  {
    [velocityHisto reset];
    [velocityHisto update];
    [velocityHisto output];
  }  
  if(populationHisto)
  {
    [populationHisto reset];
    [populationHisto update];
    [populationHisto output];
  }  
  if(mortalityGraph) 
  {
     [mortalityGraph step];
     [mortalityGraph update];
     [mortalityGraph outputGraph];
  }


   if(habitatRasterList)
   {
       int habRasterCount = [habitatRasterList getCount];
       int cellDisplayCount = [habCellDisplayList getCount];
       int barrierDisplayCount = [habBarrierDisplayList getCount];
       int i;
    
       id aRaster = nil;
       id aCellDisplay = nil;
       id aBarrierDisplay = nil;

       if((habRasterCount != cellDisplayCount) && (habRasterCount != barrierDisplayCount))
       {
           fprintf(stderr, "ERROR: TroutObserverSwarm >>>> Display counts unmatched\n");
           fflush(0);
           exit(1);
       }


       for(i = 0; i < habRasterCount; i++)
       {
          if((aRaster = [habitatRasterList atOffset: i]) != nil)
          if((aCellDisplay = [habCellDisplayList atOffset: i]) != nil)
          {
             aBarrierDisplay = [habBarrierDisplayList atOffset: i];

             [aRaster erase];
             [aCellDisplay display];
             [aBarrierDisplay display];
             [aRaster drawSelf];
          }

       }
    } //if habitatRasterList
       

  return self;
}

- buildActions
{
  [super buildActions];
  [troutModelSwarm buildActions];

  displayActions = [ActionGroup create: obsZone];
  [displayActions createActionTo: self message: M(_update_)];
  [displayActions createActionTo: probeDisplayManager message: M(update)];


  displaySchedule = [Schedule createBegin: obsZone];
  [displaySchedule setRepeatInterval: displayFrequency]; // note frequency!
  displaySchedule = [displaySchedule createEnd];
  [displaySchedule at: 0 createAction: displayActions];
  [displaySchedule at: 0 createActionTo: self message: M(checkToStop)];
  
  if (rasterOutputFrequency > 0) {
    outputSchedule = [Schedule createBegin: obsZone];
    [outputSchedule setRepeatInterval: rasterOutputFrequency];
    outputSchedule = [outputSchedule createEnd];
    if ((strcmp(takeRasterPictures, "YES") == 0) ||
	(strcmp(takeRasterPictures, "yes") == 0) ||
	(strcmp(takeRasterPictures, "Yes") == 0) ||
	(strcmp(takeRasterPictures, "Y") == 0) ||
	(strcmp(takeRasterPictures, "y") == 0))
      [outputSchedule at: 0 createActionTo: self message: M(writeFrame)];
  }

  return self;
}  


/////////////////////////////////////////
//
// setExperSwarm
//
/////////////////////////////////////////
- setExperSwarm: anExperSwarm
{
    experSwarm = anExperSwarm;
    return self;
}

///////////////////////////////////////////////
//
// updateTkEvents
//
///////////////////////////////////////////////
- updateTkEvents
{


   if(experSwarm == nil)
   {
       fprintf(stderr, "ERROR: TroutObserverSwarm >>>> updateTkEvents >>>> experSwarm is nil\n");
       fflush(0);
       exit(1);
   }
   if(habitatRasterList)
   {
       int habRasterCount = [habitatRasterList getCount];
       int cellDisplayCount = [habCellDisplayList getCount];
       int barrierDisplayCount = [habBarrierDisplayList getCount];
       int i;
    
       id aRaster = nil;
       id aCellDisplay = nil;
       id aBarrierDisplay = nil;

       if((habRasterCount != cellDisplayCount) && (habRasterCount != barrierDisplayCount))
       {
           fprintf(stderr, "ERROR: TroutObserverSwarm >>>> Display counts unmatched\n");
           fflush(0);
           exit(1);
       }


       for(i = 0; i < habRasterCount; i++)
       {
          if((aRaster = [habitatRasterList atOffset: i]) != nil)
          if((aCellDisplay = [habCellDisplayList atOffset: i]) != nil)
          {
             aBarrierDisplay = [habBarrierDisplayList atOffset: i];

             [aRaster erase];
             [aCellDisplay display];
             [aBarrierDisplay display];
             [aRaster drawSelf];
          }
       }
    } //if habitatRasterL

   [experSwarm updateTkEvents];
   return self;
}



/////////////////////////////////////////////////////
//
// getWorldRaster
//
////////////////////////////////////////////////////
- (id <ZoomRaster>) getWorldRaster 
{
  return worldRaster;
}


/////////////////////////////////////////////
//
// activateIn
//
////////////////////////////////////////////
- activateIn:  swarmContext
{

  fprintf(stderr, "OBSERVER SWARM >>>> activateIn begin\n");
  fprintf(stderr, "OBSERVER SWARM >>>> activateIn begin super = %s\n", [super getName]);
  fprintf(stderr, "OBSERVER SWARM >>>> activateIn begin swarmContext = %p\n", swarmContext);
  //fprintf(stderr, "OBSERVER SWARM >>>> activateIn begin actionCache = %p\n", actionCache);
  fflush(0);

  [super activateIn: swarmContext];
  modelActivity = [troutModelSwarm activateIn: self];
  fprintf(stderr, "OBSERVER SWARM >>>> activateIn modelActivity = %p\n", modelActivity);
  fflush(0);
  [displaySchedule activateIn: self];
  fprintf(stderr, "OBSERVER SWARM >>>> activateIn displaySchedule = %p\n", displaySchedule);
  fflush(0);
  if (rasterOutputFrequency > 0)
  {
    [outputSchedule activateIn: self];
  }

  myActivity = [self getActivity];

  fprintf(stderr, "OBSERVER SWARM >>>> activateIn returning myActivity = %p\n", myActivity);
  fflush(0);

  return [self getActivity];
}



////////////////////////////////
//
// checkToStop
//
////////////////////////////////
- checkToStop
{

  if([troutModelSwarm whenToStop] == YES) 
  {
    finished = YES;
    modelActivity = nil;
    [[self getActivity] stop];
 
    fprintf(stdout,"TroutObserverSwarm >>>> Stop date achieved\n");
    fflush(0);

  }


  return self;
}



- (BOOL) areYouFinishedYet 
{
  return finished;
}

- setModelNumberTo: (int) anInt 
{
  modelNumber = anInt;
  return self;
}

-(void) writeFrame 
{
  char filename[256];
  id pixID;
  id raster = nil;

  if([habitatRasterList getCount] > 0)
  {
      raster = [habitatRasterList getFirst];
 
      if(raster == nil)
      {
         fprintf(stderr, "ERROR: TroutModelSwarm >>>> writeFrame >>>> raster is nil\n");
         fflush(0);
         exit(1);
      }

      sprintf(filename, "Model%03d_Frame%03ld.png", modelNumber, getCurrentTime());

      pixID =  [Pixmap createBegin: [self getZone]];
      [pixID  setWidget: raster];
      pixID = [pixID createEnd];
      [pixID save: filename];
      [pixID drop];
  }

}


- iAmAlive 
{
  static int iveBeenCalled=0;
  iveBeenCalled++;
  (void) fprintf(stdout, "ObserverSwarm is alive. (%d)\n", iveBeenCalled); 
  fflush(0);
  return self;
}

- (void) drop 
{
  fprintf(stderr,"Observerswarm >>>> drop >>>> BEGIN\n");
  fflush(stderr);

  [probeDisplayManager setDropImmediatelyFlag: NO];

  if(habitatRasterList)
  {
      [habitatRasterList deleteAll];
  }
  if(habCellDisplayList)
  {
      [habCellDisplayList deleteAll];
  }
  if(habBarrierDisplayList)
  {
      [habBarrierDisplayList deleteAll];
  }
  if(velocityHisto)
  {
      [velocityHisto drop];
      velocityHisto = nil;
  }

  if(depthHisto) 
  {
     [depthHisto drop];
     depthHisto = nil;
  }

  if(populationHisto)
  {
     [populationHisto drop];
     populationHisto = nil;
  }

  if(mortalityGraph)
  {
      [mortalityGraph drop];
      mortalityGraph = nil;
  }


  if(displayActions)
  {
     [displayActions drop];
     displayActions = nil;
  }

  if(displaySchedule)
  {
      [displaySchedule drop];
      displaySchedule = nil;
  }

  //[[troutModelSwarm getActivity] drop];

  if(troutModelSwarm)
  {
      fprintf(stderr, "OBSERVER SWARM >>>> drop >>>> dropping troutModelSwarm\n");
      fflush(0);

      [troutModelSwarm drop];
      troutModelSwarm = nil;
  }

  if(probeMap)
  {
      [probeMap drop];
      probeMap = nil;
  }

  if(obsZone)
  {
      [obsZone drop];
      obsZone = nil;
  }
  
  [super drop];

  fprintf(stderr,"Observerswarm >>>> drop >>>> END\n");
  fflush(stderr);
  

} //drop


- (id <Swarm>) getModelSwarm 
{
     return troutModelSwarm;
}

@end
